@@ -2,11 +2,9 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library server.operation;
-
 import 'dart:async';
 
-import 'package:analysis_server/plugin/protocol/protocol.dart';
+import 'package:analysis_server/protocol/protocol_generated.dart';
 import 'package:logging/logging.dart';
 
 import 'driver.dart';
@@ -133,7 +131,7 @@ class ResponseOperation extends Operation {
   bool _equal(expectedResult, actualResult) {
     if (expectedResult is Map && actualResult is Map) {
       if (expectedResult.length == actualResult.length) {
-        return expectedResult.keys.every((String key) {
+        return expectedResult.keys.every((key) {
           return key ==
                   'fileStamp' || // fileStamp values will not be the same across runs
               _equal(expectedResult[key], actualResult[key]);
@@ -222,7 +220,7 @@ class WaitForAnalysisCompleteOperation extends Operation {
         completer.complete();
         return;
       }
-      // Timeout if no communcation received within the last 60 seconds.
+      // Timeout if no communication received within the last 60 seconds.
       double currentTime = driver.server.currentElapseTime;
       double lastTime = driver.server.lastCommunicationTime;
       if (currentTime - lastTime > 60) {
