@@ -2,15 +2,14 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library analysis_server.plugin.edit.fix.fix_core;
-
 import 'dart:async';
 
-import 'package:analysis_server/plugin/protocol/protocol.dart'
-    show SourceChange;
 import 'package:analyzer/error/error.dart';
 import 'package:analyzer/file_system/file_system.dart';
-import 'package:analyzer/src/generated/engine.dart';
+import 'package:analyzer/src/dart/analysis/driver.dart';
+import 'package:analyzer_plugin/protocol/protocol_common.dart'
+    show SourceChange;
+import 'package:analyzer_plugin/utilities/fixes/fixes.dart';
 
 /**
  * A description of a single proposed fix for some problem.
@@ -25,11 +24,15 @@ class Fix {
 
   /**
    * A comparator that can be used to sort fixes by their relevance. The most
-   * relevant fixes will be sorted before fixes with a lower relevance.
+   * relevant fixes will be sorted before fixes with a lower relevance. Fixes
+   * with the same relevance are sorted alphabetically.
    */
-  static final Comparator<Fix> SORT_BY_RELEVANCE =
-      (Fix firstFix, Fix secondFix) =>
-          firstFix.kind.relevance - secondFix.kind.relevance;
+  static final Comparator<Fix> SORT_BY_RELEVANCE = (Fix a, Fix b) {
+    if (a.kind.priority != b.kind.priority) {
+      return a.kind.priority - b.kind.priority;
+    }
+    return a.change.message.compareTo(b.change.message);
+  };
 
   /**
    * A description of the fix being proposed.
@@ -46,6 +49,11 @@ class Fix {
    */
   Fix(this.kind, this.change);
 
+  /**
+   * Returns `true` if this fix is the union of multiple fixes.
+   */
+  bool isFixAllFix() => change.message == kind.appliedTogetherMessage;
+
   @override
   String toString() {
     return 'Fix(kind=$kind, change=$change)';
@@ -59,15 +67,21 @@ class Fix {
  */
 abstract class FixContext {
   /**
-   * The [AnalysisContext] to get fixes in.
+   * The analysis driver used to access analysis results.
    */
-  AnalysisContext get analysisContext;
+  AnalysisDriver get analysisDriver;
 
   /**
-   * The error to fix, should be reported in the given [analysisContext].
+   * The error to fix, should be reported by the given [analysisDriver].
    */
   AnalysisError get error;
 
+  /**
+   * All of the errors in the file. This is used to compute additional fixes
+   * such "Fix all instances in file."
+   */
+  List<AnalysisError> get errors;
+
   /**
    * The [ResourceProvider] to access files and folders.
    */
@@ -87,40 +101,3 @@ abstract class FixContributor {
    */
   Future<List<Fix>> computeFixes(FixContext context);
 }
-
-/**
- * A description of a class of fixes. Instances are intended to hold the
- * information that is common across a number of fixes and to be shared by those
- * fixes. For example, if an unnecessary cast is found then one of the suggested
- * fixes will be to remove the cast. If there are multiple unnecessary casts in
- * a single file, then there will be multiple fixes, one per occurrence, but
- * they will all share the same kind.
- *
- * Clients may not extend, implement or mix-in this class.
- */
-class FixKind {
-  /**
-   * The name of this kind of fix, used for debugging.
-   */
-  final String name;
-
-  /**
-   * The relevance of this kind of fix for the kind of error being addressed.
-   */
-  final int relevance;
-
-  /**
-   * A human-readable description of the changes that will be applied by this
-   * kind of fix.
-   */
-  final String message;
-
-  /**
-   * Initialize a newly created kind of fix to have the given [name],
-   * [relevance] and [message].
-   */
-  const FixKind(this.name, this.relevance, this.message);
-
-  @override
-  String toString() => name;
-}
