@@ -2,36 +2,35 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library domain.completion;
-
 import 'dart:async';
 
-import 'package:analysis_server/plugin/protocol/protocol.dart';
+import 'package:analysis_server/protocol/protocol.dart';
+import 'package:analysis_server/protocol/protocol_constants.dart';
+import 'package:analysis_server/protocol/protocol_generated.dart';
 import 'package:analysis_server/src/analysis_server.dart';
-import 'package:analysis_server/src/constants.dart';
+import 'package:analysis_server/src/collections.dart';
+import 'package:analysis_server/src/domain_abstract.dart';
+import 'package:analysis_server/src/plugin/plugin_manager.dart';
 import 'package:analysis_server/src/provisional/completion/completion_core.dart';
 import 'package:analysis_server/src/services/completion/completion_core.dart';
 import 'package:analysis_server/src/services/completion/completion_performance.dart';
+import 'package:analysis_server/src/services/completion/dart/completion_manager.dart';
 import 'package:analyzer/src/dart/analysis/driver.dart';
-import 'package:analyzer/src/generated/engine.dart' hide AnalysisResult;
-import 'package:analyzer/src/source/source_resource.dart';
-import 'package:analyzer/src/generated/source.dart';
+import 'package:analyzer_plugin/protocol/protocol.dart' as plugin;
+import 'package:analyzer_plugin/protocol/protocol_common.dart';
+import 'package:analyzer_plugin/protocol/protocol_constants.dart' as plugin;
+import 'package:analyzer_plugin/protocol/protocol_generated.dart' as plugin;
 
 /**
  * Instances of the class [CompletionDomainHandler] implement a [RequestHandler]
  * that handles requests in the completion domain.
  */
-class CompletionDomainHandler implements RequestHandler {
+class CompletionDomainHandler extends AbstractRequestHandler {
   /**
    * The maximum number of performance measurements to keep.
    */
   static const int performanceListMaxLength = 50;
 
-  /**
-   * The analysis server that is using this handler to process requests.
-   */
-  final AnalysisServer server;
-
   /**
    * The next completion response id.
    */
@@ -46,13 +45,8 @@ class CompletionDomainHandler implements RequestHandler {
    * A list of code completion performance measurements for the latest
    * completion operation up to [performanceListMaxLength] measurements.
    */
-  final List<CompletionPerformance> performanceList =
-      new List<CompletionPerformance>();
-
-  /**
-   * Performance for the last priority change event.
-   */
-  CompletionPerformance computeCachePerformance;
+  final RecentBuffer<CompletionPerformance> performanceList =
+      new RecentBuffer<CompletionPerformance>(performanceListMaxLength);
 
   /**
    * The current request being processed or `null` if none.
@@ -62,7 +56,7 @@ class CompletionDomainHandler implements RequestHandler {
   /**
    * Initialize a new request handler for the given [server].
    */
-  CompletionDomainHandler(this.server);
+  CompletionDomainHandler(AnalysisServer server) : super(server);
 
   /**
    * Compute completion results for the given request and append them to the stream.
@@ -71,45 +65,82 @@ class CompletionDomainHandler implements RequestHandler {
    * Subclasses should override this method, append at least one result
    * to the [controller], and close the controller stream once complete.
    */
-  Future<CompletionResult> computeSuggestions(
-      CompletionRequestImpl request) async {
-    Iterable<CompletionContributor> newContributors =
-        server.serverPlugin.completionContributors;
+  Future<CompletionResult> computeSuggestions(CompletionRequestImpl request,
+      CompletionGetSuggestionsParams params) async {
+    // TODO(brianwilkerson) Determine whether this await is necessary.
+    await null;
+    //
+    // Allow plugins to start computing fixes.
+    //
+    Map<PluginInfo, Future<plugin.Response>> pluginFutures;
+    plugin.CompletionGetSuggestionsParams requestParams;
+    String file = params.file;
+    int offset = params.offset;
+    AnalysisDriver driver = server.getAnalysisDriver(file);
+    if (driver != null) {
+      requestParams = new plugin.CompletionGetSuggestionsParams(file, offset);
+      pluginFutures = server.pluginManager
+          .broadcastRequest(requestParams, contextRoot: driver.contextRoot);
+    }
+    //
+    // Compute completions generated by server.
+    //
     List<CompletionSuggestion> suggestions = <CompletionSuggestion>[];
+    if (request.result != null) {
+      const COMPUTE_SUGGESTIONS_TAG = 'computeSuggestions';
+      performance.logStartTime(COMPUTE_SUGGESTIONS_TAG);
 
-    const COMPUTE_SUGGESTIONS_TAG = 'computeSuggestions';
-    performance.logStartTime(COMPUTE_SUGGESTIONS_TAG);
-
-    for (CompletionContributor contributor in newContributors) {
-      String contributorTag = 'computeSuggestions - ${contributor.runtimeType}';
+      CompletionContributor contributor = new DartCompletionManager();
+      String contributorTag = 'computeSuggestions - ${contributor
+          .runtimeType}';
       performance.logStartTime(contributorTag);
       try {
         suggestions.addAll(await contributor.computeSuggestions(request));
       } on AbortCompletion {
         suggestions.clear();
-        break;
       }
       performance.logElapseTime(contributorTag);
+      performance.logElapseTime(COMPUTE_SUGGESTIONS_TAG);
     }
+    // TODO (danrubel) if request is obsolete (processAnalysisRequest returns
+    // false) then send empty results
 
-    performance.logElapseTime(COMPUTE_SUGGESTIONS_TAG);
-
-    // TODO (danrubel) if request is obsolete
-    // (processAnalysisRequest returns false)
-    // then send empty results
-
+    //
+    // Add the fixes produced by plugins to the server-generated fixes.
+    //
+    if (pluginFutures != null) {
+      List<plugin.Response> responses = await waitForResponses(pluginFutures,
+          requestParameters: requestParams);
+      for (plugin.Response response in responses) {
+        plugin.CompletionGetSuggestionsResult result =
+            new plugin.CompletionGetSuggestionsResult.fromResponse(response);
+        if (result.results != null && result.results.isNotEmpty) {
+          if (suggestions.isEmpty) {
+            request.replacementOffset = result.replacementOffset;
+            request.replacementLength = result.replacementLength;
+          } else if (request.replacementOffset != result.replacementOffset &&
+              request.replacementLength != result.replacementLength) {
+            server.instrumentationService
+                .logError('Plugin completion-results dropped due to conflicting'
+                    ' replacement offset/length: ${result.toJson()}');
+            continue;
+          }
+          suggestions.addAll(result.results);
+        }
+      }
+    }
+    //
+    // Return the result.
+    //
     return new CompletionResult(
         request.replacementOffset, request.replacementLength, suggestions);
   }
 
   @override
   Response handleRequest(Request request) {
-    if (server.searchEngine == null) {
-      return new Response.noIndexGenerated(request);
-    }
     return runZoned(() {
       String requestName = request.method;
-      if (requestName == COMPLETION_GET_SUGGESTIONS) {
+      if (requestName == COMPLETION_REQUEST_GET_SUGGESTIONS) {
         processRequest(request);
         return Response.DELAYED_RESPONSE;
       }
@@ -122,97 +153,73 @@ class CompletionDomainHandler implements RequestHandler {
     });
   }
 
+  void ifMatchesRequestClear(CompletionRequest completionRequest) {
+    if (_currentRequest == completionRequest) {
+      _currentRequest = null;
+    }
+  }
+
   /**
    * Process a `completion.getSuggestions` request.
    */
   Future<Null> processRequest(Request request) async {
+    // TODO(brianwilkerson) Determine whether this await is necessary.
+    await null;
     performance = new CompletionPerformance();
 
     // extract and validate params
     CompletionGetSuggestionsParams params =
         new CompletionGetSuggestionsParams.fromRequest(request);
+    String filePath = params.file;
+    int offset = params.offset;
 
-    AnalysisResult result;
-    AnalysisContext context;
-    Source source;
-    if (server.options.enableNewAnalysisDriver) {
-      result = await server.getAnalysisResult(params.file);
+    AnalysisResult result = await server.getAnalysisResult(filePath);
 
-      if (result == null || !result.exists) {
-        server.sendResponse(new Response.unknownSource(request));
+    if (result == null || !result.exists) {
+      if (server.onNoAnalysisCompletion != null) {
+        String completionId = (_nextCompletionId++).toString();
+        await server.onNoAnalysisCompletion(
+            request, this, params, performance, completionId);
         return;
       }
-
-      if (params.offset < 0 || params.offset > result.content.length) {
-        server.sendResponse(new Response.invalidParameter(
-            request,
-            'params.offset',
-            'Expected offset between 0 and source length inclusive,'
-            ' but found ${params.offset}'));
-        return;
-      }
-
-      source = new FileSource(
-          server.resourceProvider.getFile(result.path), result.uri);
     } else {
-      ContextSourcePair contextSource =
-          server.getContextSourcePair(params.file);
-
-      context = contextSource.context;
-      source = contextSource.source;
-      if (context == null || !context.exists(source)) {
-        server.sendResponse(new Response.unknownSource(request));
-        return;
-      }
-
-      TimestampedData<String> contents = context.getContents(source);
-      if (params.offset < 0 || params.offset > contents.data.length) {
+      if (offset < 0 || offset > result.content.length) {
         server.sendResponse(new Response.invalidParameter(
             request,
             'params.offset',
             'Expected offset between 0 and source length inclusive,'
-            ' but found ${params.offset}'));
+            ' but found $offset'));
         return;
       }
+
+      recordRequest(performance, filePath, result.content, offset);
     }
+    CompletionRequestImpl completionRequest =
+        new CompletionRequestImpl(result, offset, performance);
 
-    recordRequest(performance, context, source, params.offset);
-
-    CompletionRequestImpl completionRequest = new CompletionRequestImpl(
-        result,
-        context,
-        server.resourceProvider,
-        server.searchEngine,
-        source,
-        params.offset,
-        performance,
-        server.ideOptions);
     String completionId = (_nextCompletionId++).toString();
 
-    _abortCurrentRequest();
-    _currentRequest = completionRequest;
+    setNewRequest(completionRequest);
 
     // initial response without results
     server.sendResponse(new CompletionGetSuggestionsResult(completionId)
         .toResponse(request.id));
 
     // Compute suggestions in the background
-    computeSuggestions(completionRequest).then((CompletionResult result) {
+    computeSuggestions(completionRequest, params)
+        .then((CompletionResult result) {
       const SEND_NOTIFICATION_TAG = 'send notification';
       performance.logStartTime(SEND_NOTIFICATION_TAG);
       sendCompletionNotification(completionId, result.replacementOffset,
           result.replacementLength, result.suggestions);
       performance.logElapseTime(SEND_NOTIFICATION_TAG);
-
       performance.notificationCount = 1;
       performance.logFirstNotificationComplete('notification 1 complete');
       performance.suggestionCountFirst = result.suggestions.length;
       performance.suggestionCountLast = result.suggestions.length;
       performance.complete();
     }).whenComplete(() {
-      if (_currentRequest == completionRequest) {
-        _currentRequest = null;
-      }
+      ifMatchesRequestClear(completionRequest);
     });
   }
 
@@ -220,20 +227,13 @@ class CompletionDomainHandler implements RequestHandler {
    * If tracking code completion performance over time, then
    * record addition information about the request in the performance record.
    */
-  void recordRequest(CompletionPerformance performance, AnalysisContext context,
-      Source source, int offset) {
-    performance.source = source;
-    if (performanceListMaxLength == 0 || context == null || source == null) {
+  void recordRequest(CompletionPerformance performance, String path,
+      String content, int offset) {
+    performance.path = path;
+    if (performanceListMaxLength == 0) {
       return;
     }
-    TimestampedData<String> data = context.getContents(source);
-    if (data == null) {
-      return;
-    }
-    performance.setContentsAndOffset(data.data, offset);
-    while (performanceList.length >= performanceListMaxLength) {
-      performanceList.removeAt(0);
-    }
+    performance.setContentsAndOffset(content, offset);
     performanceList.add(performance);
   }
 
@@ -247,6 +247,11 @@ class CompletionDomainHandler implements RequestHandler {
         .toNotification());
   }
 
+  void setNewRequest(CompletionRequest completionRequest) {
+    _abortCurrentRequest();
+    _currentRequest = completionRequest;
+  }
+
   /**
    * Abort the current completion request, if any.
    */
