@@ -2,10 +2,8 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library protocol.server;
-
-import 'package:analysis_server/plugin/protocol/protocol.dart';
 import 'package:analysis_server/plugin/protocol/protocol_dart.dart';
+import 'package:analysis_server/protocol/protocol_generated.dart';
 import 'package:analysis_server/src/services/correction/fix.dart';
 import 'package:analysis_server/src/services/search/search_engine.dart'
     as engine;
@@ -16,14 +14,18 @@ import 'package:analyzer/dart/element/type.dart' as engine;
 import 'package:analyzer/error/error.dart' as engine;
 import 'package:analyzer/exception/exception.dart';
 import 'package:analyzer/source/error_processor.dart';
+import 'package:analyzer/source/line_info.dart';
 import 'package:analyzer/src/dart/ast/utilities.dart' as engine;
 import 'package:analyzer/src/error/codes.dart' as engine;
 import 'package:analyzer/src/generated/engine.dart' as engine;
 import 'package:analyzer/src/generated/source.dart' as engine;
 import 'package:analyzer/src/generated/utilities_dart.dart' as engine;
+import 'package:analyzer_plugin/protocol/protocol_common.dart';
 
-export 'package:analysis_server/plugin/protocol/protocol.dart';
 export 'package:analysis_server/plugin/protocol/protocol_dart.dart';
+export 'package:analysis_server/protocol/protocol.dart';
+export 'package:analysis_server/protocol/protocol_generated.dart';
+export 'package:analyzer_plugin/protocol/protocol_common.dart';
 
 /**
  * Returns a list of AnalysisErrors corresponding to the given list of Engine
@@ -106,7 +108,7 @@ AnalysisError newAnalysisError_fromEngine(
     int startLine = -1;
     int startColumn = -1;
     if (lineInfo != null) {
-      engine.LineInfo_Location lineLocation = lineInfo.getLocation(offset);
+      CharacterLocation lineLocation = lineInfo.getLocation(offset);
       if (lineLocation != null) {
         startLine = lineLocation.lineNumber;
         startColumn = lineLocation.columnNumber;
@@ -271,8 +273,7 @@ Location _locationForArgs(
   try {
     engine.LineInfo lineInfo = unitElement.lineInfo;
     if (lineInfo != null) {
-      engine.LineInfo_Location offsetLocation =
-          lineInfo.getLocation(range.offset);
+      CharacterLocation offsetLocation = lineInfo.getLocation(range.offset);
       startLine = offsetLocation.lineNumber;
       startColumn = offsetLocation.columnNumber;
     }
