@@ -2,20 +2,17 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library services.completion.contributor.dart.arglist;
-
 import 'dart:async';
 
-import 'package:analysis_server/src/ide_options.dart';
 import 'package:analysis_server/src/protocol_server.dart'
     hide Element, ElementKind;
 import 'package:analysis_server/src/provisional/completion/dart/completion_dart.dart';
 import 'package:analysis_server/src/services/completion/dart/utilities.dart';
-import 'package:analysis_server/src/services/correction/flutter_util.dart';
 import 'package:analysis_server/src/utilities/documentation.dart';
+import 'package:analysis_server/src/utilities/flutter.dart' as flutter;
 import 'package:analyzer/dart/ast/ast.dart';
+import 'package:analyzer/dart/ast/token.dart';
 import 'package:analyzer/dart/element/element.dart';
-import 'package:analyzer/src/generated/utilities_dart.dart';
 
 /**
  * Determine the number of arguments.
@@ -25,7 +22,7 @@ int _argCount(DartCompletionRequest request) {
   if (node is ArgumentList) {
     if (request.target.entity == node.rightParenthesis) {
       // Parser ignores trailing commas
-      if (node.rightParenthesis.previous?.lexeme == ',') {
+      if (node.findPrevious(node.rightParenthesis)?.lexeme == ',') {
         return node.arguments.length + 1;
       }
     }
@@ -182,25 +179,17 @@ class ArgListContributor extends DartCompletionContributor {
   @override
   Future<List<CompletionSuggestion>> computeSuggestions(
       DartCompletionRequest request) async {
+    // TODO(brianwilkerson) Determine whether this await is necessary.
+    await null;
     this.request = request;
     this.suggestions = <CompletionSuggestion>[];
 
     // Determine if the target is in an argument list
     // for a method or a constructor or an annotation
-    // and resolve the identifier
     SimpleIdentifier targetId = _getTargetId(request.target.containingNode);
     if (targetId == null) {
       return EMPTY_LIST;
     }
-
-    // Resolve the target expression to determine the arguments
-    await request.resolveContainingExpression(targetId);
-    // Gracefully degrade if the element could not be resolved
-    // e.g. target changed, completion aborted
-    targetId = _getTargetId(request.target.containingNode);
-    if (targetId == null) {
-      return EMPTY_LIST;
-    }
     Element elem = targetId.bestElement;
     if (elem == null) {
       return EMPTY_LIST;
@@ -231,19 +220,15 @@ class ArgListContributor extends DartCompletionContributor {
     bool appendColon = !_isInNamedExpression(request);
     Iterable<String> namedArgs = _namedArgs(request);
     for (ParameterElement parameter in parameters) {
-      if (parameter.parameterKind == ParameterKind.NAMED) {
+      if (parameter.isNamed) {
         _addNamedParameterSuggestion(
-            request, namedArgs, parameter, appendColon, appendComma);
+            namedArgs, parameter, appendColon, appendComma);
       }
     }
   }
 
-  void _addNamedParameterSuggestion(
-      DartCompletionRequest request,
-      List<String> namedArgs,
-      ParameterElement parameter,
-      bool appendColon,
-      bool appendComma) {
+  void _addNamedParameterSuggestion(List<String> namedArgs,
+      ParameterElement parameter, bool appendColon, bool appendComma) {
     String name = parameter.name;
     String type = parameter.type?.displayName;
     if (name != null && name.length > 0 && !namedArgs.contains(name)) {
@@ -251,14 +236,33 @@ class ArgListContributor extends DartCompletionContributor {
       if (appendColon) {
         completion += ': ';
       }
+      int selectionOffset = completion.length;
+
+      // Optionally add Flutter child widget details.
+      Element element = parameter.enclosingElement;
+      if (element is ConstructorElement) {
+        if (flutter.isWidget(element.enclosingElement)) {
+          String value = getDefaultStringParameterValue(parameter);
+          if (value == '<Widget>[]') {
+            completion += value;
+            selectionOffset = completion.length - 1; // before closing ']'
+          }
+        }
+      }
+
       if (appendComma) {
         completion += ',';
       }
+
+      final int relevance = parameter.hasRequired
+          ? DART_RELEVANCE_NAMED_PARAMETER_REQUIRED
+          : DART_RELEVANCE_NAMED_PARAMETER;
+
       CompletionSuggestion suggestion = new CompletionSuggestion(
           CompletionSuggestionKind.NAMED_ARGUMENT,
-          DART_RELEVANCE_NAMED_PARAMETER,
+          relevance,
           completion,
-          completion.length,
+          selectionOffset,
           0,
           false,
           false,
@@ -269,42 +273,16 @@ class ArgListContributor extends DartCompletionContributor {
         suggestion.element = convertElement(parameter);
       }
 
-      String defaultValue = _getDefaultValue(parameter, request.ideOptions);
-      if (defaultValue != null) {
-        StringBuffer sb = new StringBuffer();
-        sb.write('${parameter.name}: ');
-        int offset = sb.length;
-        sb.write(defaultValue);
-        suggestion.defaultArgumentListString = sb.toString();
-        suggestion.defaultArgumentListTextRanges = [
-          offset,
-          defaultValue.length
-        ];
-      }
-
       suggestions.add(suggestion);
     }
   }
 
-  String _getDefaultValue(ParameterElement param, IdeOptions options) {
-    if (options?.generateFlutterWidgetChildrenBoilerPlate == true) {
-      Element element = param.enclosingElement;
-      if (element is ConstructorElement) {
-        if (isFlutterWidget(element.enclosingElement) &&
-            param.name == 'children') {
-          return getDefaultStringParameterValue(param);
-        }
-      }
-    }
-    return null;
-  }
-
   void _addSuggestions(Iterable<ParameterElement> parameters) {
     if (parameters == null || parameters.length == 0) {
       return;
     }
-    Iterable<ParameterElement> requiredParam = parameters.where(
-        (ParameterElement p) => p.parameterKind == ParameterKind.REQUIRED);
+    Iterable<ParameterElement> requiredParam =
+        parameters.where((ParameterElement p) => p.isNotOptional);
     int requiredCount = requiredParam.length;
     // TODO (jwren) _isAppendingToArgList can be split into two cases (with and
     // without preceded), then _isAppendingToArgList,
@@ -313,15 +291,40 @@ class ArgListContributor extends DartCompletionContributor {
     // method which returns some enum with 5+ cases.
     if (_isEditingNamedArgLabel(request) || _isAppendingToArgList(request)) {
       if (requiredCount == 0 || requiredCount < _argCount(request)) {
-        _addDefaultParamSuggestions(parameters);
+        bool addTrailingComma =
+            !_isFollowedByAComma(request) && _isInFlutterCreation(request);
+        _addDefaultParamSuggestions(parameters, addTrailingComma);
       }
     } else if (_isInsertingToArgListWithNoSynthetic(request)) {
       _addDefaultParamSuggestions(parameters, true);
     } else if (_isInsertingToArgListWithSynthetic(request)) {
-      _addDefaultParamSuggestions(parameters);
+      _addDefaultParamSuggestions(parameters, !_isFollowedByAComma(request));
     }
   }
 
+  bool _isFollowedByAComma(DartCompletionRequest request) {
+    // new A(^); NO
+    // new A(one: 1, ^); NO
+    // new A(^ , one: 1); YES
+    // new A(^), ... NO
+
+    var containingNode = request.target.containingNode;
+    var entity = request.target.entity;
+    Token token =
+        entity is AstNode ? entity.endToken : entity is Token ? entity : null;
+    return (token != containingNode?.endToken) &&
+        token?.next?.type == TokenType.COMMA &&
+        !token.next.isSynthetic;
+  }
+
+  bool _isInFlutterCreation(DartCompletionRequest request) {
+    AstNode containingNode = request?.target?.containingNode;
+    InstanceCreationExpression newExpr = containingNode != null
+        ? flutter.identifyNewExpression(containingNode.parent)
+        : null;
+    return newExpr != null && flutter.isWidgetCreation(newExpr);
+  }
+
   /**
    * If the given [comment] is not `null`, fill the [suggestion] documentation
    * fields.
