@@ -1,26 +1,25 @@
 // Copyright (c) 2014, the Dart project authors.  Please see the AUTHORS file
-
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library services.completion.dart.keyword;
-
 import 'dart:async';
 
-import 'package:analysis_server/plugin/protocol/protocol.dart';
 import 'package:analysis_server/src/provisional/completion/dart/completion_dart.dart';
 import 'package:analysis_server/src/services/completion/dart/completion_manager.dart';
-import 'package:analysis_server/src/services/completion/dart/optype.dart';
 import 'package:analyzer/dart/ast/ast.dart';
 import 'package:analyzer/dart/ast/token.dart';
 import 'package:analyzer/dart/ast/visitor.dart';
 import 'package:analyzer/src/dart/ast/token.dart';
+import 'package:analyzer_plugin/protocol/protocol_common.dart';
+import 'package:analyzer_plugin/src/utilities/completion/optype.dart';
 
-const ASYNC = 'async';
 const ASYNC_STAR = 'async*';
-const AWAIT = 'await';
+const DEFAULT_COLON = 'default:';
+const DEFERRED_AS = 'deferred as';
+const EXPORT_STATEMENT = "export '';";
+const IMPORT_STATEMENT = "import '';";
+const PART_STATEMENT = "part '';";
 const SYNC_STAR = 'sync*';
-const YIELD = 'yield';
 const YIELD_STAR = 'yield*';
 
 /**
@@ -31,7 +30,15 @@ class KeywordContributor extends DartCompletionContributor {
   @override
   Future<List<CompletionSuggestion>> computeSuggestions(
       DartCompletionRequest request) async {
+    // TODO(brianwilkerson) Determine whether this await is necessary.
+    await null;
     List<CompletionSuggestion> suggestions = <CompletionSuggestion>[];
+
+    // Don't suggest anything right after double or integer literals.
+    if (request.target.isDoubleOrIntLiteral()) {
+      return suggestions;
+    }
+
     request.target.containingNode
         .accept(new _KeywordVisitor(request, suggestions));
     return suggestions;
@@ -50,6 +57,12 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
       : this.request = request,
         this.entity = request.target.entity;
 
+  Token get droppedToken => request.target.droppedToken;
+
+  bool isEmptyBody(FunctionBody body) =>
+      body is EmptyFunctionBody ||
+      (body is BlockFunctionBody && body.beginToken.isSynthetic);
+
   @override
   visitArgumentList(ArgumentList node) {
     if (request is DartCompletionRequestImpl) {
@@ -61,18 +74,30 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
     }
     if (entity == node.rightParenthesis) {
       _addExpressionKeywords(node);
-      Token previous = (entity as Token).previous;
+      Token previous = node.findPrevious(entity as Token);
       if (previous.isSynthetic) {
-        previous = previous.previous;
+        previous = node.findPrevious(previous);
       }
       if (previous.lexeme == ')') {
-        _addSuggestion2(ASYNC);
+        _addSuggestion(Keyword.ASYNC);
         _addSuggestion2(ASYNC_STAR);
         _addSuggestion2(SYNC_STAR);
       }
     }
     if (entity is SimpleIdentifier && node.arguments.contains(entity)) {
       _addExpressionKeywords(node);
+      int index = node.arguments.indexOf(entity);
+      if (index > 0) {
+        Expression previousArgument = node.arguments[index - 1];
+        Token endToken = previousArgument?.endToken;
+        if (endToken?.lexeme == ')' &&
+            endToken.next?.lexeme == ',' &&
+            endToken.next.isSynthetic) {
+          _addSuggestion(Keyword.ASYNC);
+          _addSuggestion2(ASYNC_STAR);
+          _addSuggestion2(SYNC_STAR);
+        }
+      }
     }
   }
 
@@ -80,7 +105,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
   visitAsExpression(AsExpression node) {
     if (identical(entity, node.asOperator) &&
         node.expression is ParenthesizedExpression) {
-      _addSuggestion2(ASYNC, relevance: DART_RELEVANCE_HIGH);
+      _addSuggestion(Keyword.ASYNC, DART_RELEVANCE_HIGH);
       _addSuggestion2(ASYNC_STAR, relevance: DART_RELEVANCE_HIGH);
       _addSuggestion2(SYNC_STAR, relevance: DART_RELEVANCE_HIGH);
     }
@@ -91,7 +116,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
     Statement prevStmt = OpType.getPreviousStatement(node, entity);
     if (prevStmt is TryStatement) {
       if (prevStmt.finallyBlock == null) {
-        _addSuggestion2('on');
+        _addSuggestion(Keyword.ON);
         _addSuggestion(Keyword.CATCH);
         _addSuggestion(Keyword.FINALLY);
         if (prevStmt.catchClauses.isEmpty) {
@@ -106,9 +131,9 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
       Expression expression = (entity as ExpressionStatement).expression;
       if (expression is SimpleIdentifier) {
         Token token = expression.token;
-        Token previous = token.previous;
+        Token previous = node.findPrevious(token);
         if (previous.isSynthetic) {
-          previous = previous.previous;
+          previous = node.findPrevious(previous);
         }
         Token next = token.next;
         if (next.isSynthetic) {
@@ -116,7 +141,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
         }
         if (previous.type == TokenType.CLOSE_PAREN &&
             next.type == TokenType.OPEN_CURLY_BRACKET) {
-          _addSuggestion2(ASYNC);
+          _addSuggestion(Keyword.ASYNC);
           _addSuggestion2(ASYNC_STAR);
           _addSuggestion2(SYNC_STAR);
         }
@@ -140,8 +165,8 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
       _addClassBodyKeywords();
       int index = node.members.indexOf(entity);
       ClassMember previous = index > 0 ? node.members[index - 1] : null;
-      if (previous is MethodDeclaration && previous.body is EmptyFunctionBody) {
-        _addSuggestion2(ASYNC);
+      if (previous is MethodDeclaration && isEmptyBody(previous.body)) {
+        _addSuggestion(Keyword.ASYNC);
         _addSuggestion2(ASYNC_STAR);
         _addSuggestion2(SYNC_STAR);
       }
@@ -182,18 +207,18 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
           !node.directives.any((d) => d is LibraryDirective)) {
         _addSuggestions([Keyword.LIBRARY], DART_RELEVANCE_HIGH);
       }
-      _addSuggestion2('${Keyword.IMPORT.syntax} \'\';',
+      _addSuggestion2(IMPORT_STATEMENT,
           offset: 8, relevance: DART_RELEVANCE_HIGH);
-      _addSuggestion2('${Keyword.EXPORT.syntax} \'\';',
+      _addSuggestion2(EXPORT_STATEMENT,
           offset: 8, relevance: DART_RELEVANCE_HIGH);
-      _addSuggestion2('${Keyword.PART.syntax} \'\';',
+      _addSuggestion2(PART_STATEMENT,
           offset: 6, relevance: DART_RELEVANCE_HIGH);
     }
     if (entity == null || entity is Declaration) {
       if (previousMember is FunctionDeclaration &&
           previousMember.functionExpression is FunctionExpression &&
-          previousMember.functionExpression.body is EmptyFunctionBody) {
-        _addSuggestion2(ASYNC, relevance: DART_RELEVANCE_HIGH);
+          isEmptyBody(previousMember.functionExpression.body)) {
+        _addSuggestion(Keyword.ASYNC, DART_RELEVANCE_HIGH);
         _addSuggestion2(ASYNC_STAR, relevance: DART_RELEVANCE_HIGH);
         _addSuggestion2(SYNC_STAR, relevance: DART_RELEVANCE_HIGH);
       }
@@ -201,6 +226,40 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
     }
   }
 
+  @override
+  visitConstructorDeclaration(ConstructorDeclaration node) {
+    if (node.initializers.isNotEmpty) {
+      if (entity is ConstructorInitializer) {
+        _addSuggestion(Keyword.ASSERT);
+      }
+      var last = node.initializers.last;
+      if (last == entity) {
+        Token previous = node.findPrevious(last.beginToken);
+        if (previous != null && previous.end <= request.offset) {
+          _addSuggestion(Keyword.SUPER);
+          _addSuggestion(Keyword.THIS);
+        }
+      }
+    } else {
+      Token separator = node.separator;
+      if (separator != null) {
+        var offset = request.offset;
+        if (separator.end <= offset && offset <= separator.next.offset) {
+          _addSuggestion(Keyword.ASSERT);
+          _addSuggestion(Keyword.SUPER);
+          _addSuggestion(Keyword.THIS);
+        }
+      }
+    }
+  }
+
+  @override
+  visitDefaultFormalParameter(DefaultFormalParameter node) {
+    if (entity == node.defaultValue) {
+      _addExpressionKeywords(node);
+    }
+  }
+
   @override
   visitExpression(Expression node) {
     _addExpressionKeywords(node);
@@ -213,12 +272,36 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
     }
   }
 
+  @override
+  visitFieldDeclaration(FieldDeclaration node) {
+    VariableDeclarationList fields = node.fields;
+    if (entity != fields) {
+      return;
+    }
+    NodeList<VariableDeclaration> variables = fields.variables;
+    if (variables.isEmpty || request.offset > variables.first.beginToken.end) {
+      return;
+    }
+    if (node.covariantKeyword == null) {
+      _addSuggestion(Keyword.COVARIANT);
+    }
+    if (!node.isStatic) {
+      _addSuggestion(Keyword.STATIC);
+    }
+    if (!variables.first.isConst) {
+      _addSuggestion(Keyword.CONST);
+    }
+    if (!variables.first.isFinal) {
+      _addSuggestion(Keyword.FINAL);
+    }
+  }
+
   @override
   visitForEachStatement(ForEachStatement node) {
     if (entity == node.inKeyword) {
-      Token previous = node.inKeyword.previous;
+      Token previous = node.findPrevious(node.inKeyword);
       if (previous is SyntheticStringToken && previous.lexeme == 'in') {
-        previous = previous.previous;
+        previous = node.findPrevious(previous);
       }
       if (previous != null && previous.type == TokenType.EQ) {
         _addSuggestions([
@@ -241,6 +324,14 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
     if (constructorDeclaration != null) {
       _addSuggestions([Keyword.THIS]);
     }
+    if (entity is Token && (entity as Token).type == TokenType.CLOSE_PAREN) {
+      _addSuggestion(Keyword.COVARIANT);
+    } else if (entity is FormalParameter) {
+      Token beginToken = (entity as FormalParameter).beginToken;
+      if (beginToken != null && request.target.offset == beginToken.end) {
+        _addSuggestion(Keyword.COVARIANT);
+      }
+    }
   }
 
   @override
@@ -269,7 +360,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
     if (entity == node.body) {
       FunctionBody body = node.body;
       if (!body.isAsynchronous) {
-        _addSuggestion2(ASYNC, relevance: DART_RELEVANCE_HIGH);
+        _addSuggestion(Keyword.ASYNC, DART_RELEVANCE_HIGH);
         if (body is! ExpressionFunctionBody) {
           _addSuggestion2(ASYNC_STAR, relevance: DART_RELEVANCE_HIGH);
           _addSuggestion2(SYNC_STAR, relevance: DART_RELEVANCE_HIGH);
@@ -286,9 +377,18 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
   @override
   visitIfStatement(IfStatement node) {
     if (_isPreviousTokenSynthetic(entity, TokenType.CLOSE_PAREN)) {
+      // analyzer parser
       // Actual: if (x i^)
       // Parsed: if (x) i^
       _addSuggestion(Keyword.IS, DART_RELEVANCE_HIGH);
+    } else if (entity == node.rightParenthesis) {
+      if (node.condition.endToken.next == droppedToken) {
+        // fasta parser
+        // Actual: if (x i^)
+        // Parsed: if (x)
+        //    where "i" is in the token stream but not part of the AST
+        _addSuggestion(Keyword.IS, DART_RELEVANCE_HIGH);
+      }
     } else if (entity == node.thenStatement || entity == node.elseStatement) {
       _addStatementKeywords(node);
     } else if (entity == node.condition) {
@@ -339,13 +439,13 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
   @override
   visitMethodDeclaration(MethodDeclaration node) {
     if (entity == node.body) {
-      if (node.body is EmptyFunctionBody) {
+      if (isEmptyBody(node.body)) {
         _addClassBodyKeywords();
-        _addSuggestion2(ASYNC);
+        _addSuggestion(Keyword.ASYNC);
         _addSuggestion2(ASYNC_STAR);
         _addSuggestion2(SYNC_STAR);
       } else {
-        _addSuggestion2(ASYNC, relevance: DART_RELEVANCE_HIGH);
+        _addSuggestion(Keyword.ASYNC, DART_RELEVANCE_HIGH);
         if (node.body is! ExpressionFunctionBody) {
           _addSuggestion2(ASYNC_STAR, relevance: DART_RELEVANCE_HIGH);
           _addSuggestion2(SYNC_STAR, relevance: DART_RELEVANCE_HIGH);
@@ -375,6 +475,23 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
     // ignored
   }
 
+  @override
+  visitParenthesizedExpression(ParenthesizedExpression node) {
+    Expression expression = node.expression;
+    if (expression is Identifier || expression is PropertyAccess) {
+      if (entity == node.rightParenthesis) {
+        var next = expression.endToken.next;
+        if (next == entity || next == droppedToken) {
+          // Fasta parses `if (x i^)` as `if (x ^) where the `i` is in the token
+          // stream but not part of the ParenthesizedExpression.
+          _addSuggestion(Keyword.IS, DART_RELEVANCE_HIGH);
+          return;
+        }
+      }
+    }
+    _addExpressionKeywords(node);
+  }
+
   @override
   visitPrefixedIdentifier(PrefixedIdentifier node) {
     if (entity != node.identifier) {
@@ -408,17 +525,21 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
       _addExpressionKeywords(node);
     } else if (entity == node.rightBracket) {
       if (node.members.isEmpty) {
-        _addSuggestions([Keyword.CASE, Keyword.DEFAULT], DART_RELEVANCE_HIGH);
+        _addSuggestion(Keyword.CASE, DART_RELEVANCE_HIGH);
+        _addSuggestion2(DEFAULT_COLON, relevance: DART_RELEVANCE_HIGH);
       } else {
-        _addSuggestions([Keyword.CASE, Keyword.DEFAULT]);
+        _addSuggestion(Keyword.CASE);
+        _addSuggestion2(DEFAULT_COLON);
         _addStatementKeywords(node);
       }
     }
     if (node.members.contains(entity)) {
       if (entity == node.members.first) {
-        _addSuggestions([Keyword.CASE, Keyword.DEFAULT], DART_RELEVANCE_HIGH);
+        _addSuggestion(Keyword.CASE, DART_RELEVANCE_HIGH);
+        _addSuggestion2(DEFAULT_COLON, relevance: DART_RELEVANCE_HIGH);
       } else {
-        _addSuggestions([Keyword.CASE, Keyword.DEFAULT]);
+        _addSuggestion(Keyword.CASE);
+        _addSuggestion2(DEFAULT_COLON);
         _addStatementKeywords(node);
       }
     }
@@ -429,7 +550,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
     var obj = entity;
     if (obj is CatchClause ||
         (obj is KeywordToken && obj.value() == Keyword.FINALLY)) {
-      _addSuggestion2('on');
+      _addSuggestion(Keyword.ON);
       _addSuggestion(Keyword.CATCH);
       return null;
     }
@@ -446,6 +567,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
   void _addClassBodyKeywords() {
     _addSuggestions([
       Keyword.CONST,
+      Keyword.COVARIANT,
       Keyword.DYNAMIC,
       Keyword.FACTORY,
       Keyword.FINAL,
@@ -476,6 +598,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
       Keyword.ABSTRACT,
       Keyword.CLASS,
       Keyword.CONST,
+      Keyword.COVARIANT,
       Keyword.DYNAMIC,
       Keyword.FINAL,
       Keyword.TYPEDEF,
@@ -496,7 +619,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
       _addSuggestions([Keyword.SUPER, Keyword.THIS]);
     }
     if (_inAsyncMethodOrFunction(node)) {
-      _addSuggestion2(AWAIT);
+      _addSuggestion(Keyword.AWAIT);
     }
   }
 
@@ -508,15 +631,15 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
     }
     if (!hasDeferredKeyword) {
       if (!hasAsKeyword) {
-        _addSuggestion2('deferred as', relevance: DART_RELEVANCE_HIGH);
+        _addSuggestion2(DEFERRED_AS, relevance: DART_RELEVANCE_HIGH);
       } else if (entity == node.asKeyword) {
         _addSuggestion(Keyword.DEFERRED, DART_RELEVANCE_HIGH);
       }
     }
     if (!hasDeferredKeyword || hasAsKeyword) {
       if (node.combinators.isEmpty) {
-        _addSuggestion2('show', relevance: DART_RELEVANCE_HIGH);
-        _addSuggestion2('hide', relevance: DART_RELEVANCE_HIGH);
+        _addSuggestion(Keyword.SHOW, DART_RELEVANCE_HIGH);
+        _addSuggestion(Keyword.HIDE, DART_RELEVANCE_HIGH);
       }
     }
   }
@@ -526,10 +649,10 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
       _addSuggestions([Keyword.SUPER, Keyword.THIS]);
     }
     if (_inAsyncMethodOrFunction(node)) {
-      _addSuggestion2(AWAIT);
+      _addSuggestion(Keyword.AWAIT);
     } else if (_inAsyncStarOrSyncStarMethodOrFunction(node)) {
-      _addSuggestion2(AWAIT);
-      _addSuggestion2(YIELD);
+      _addSuggestion(Keyword.AWAIT);
+      _addSuggestion(Keyword.YIELD);
       _addSuggestion2(YIELD_STAR);
     }
     if (_inLoop(node)) {
@@ -561,7 +684,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
 
   void _addSuggestion(Keyword keyword,
       [int relevance = DART_RELEVANCE_KEYWORD]) {
-    _addSuggestion2(keyword.syntax, relevance: relevance);
+    _addSuggestion2(keyword.lexeme, relevance: relevance);
   }
 
   void _addSuggestion2(String completion,
@@ -659,7 +782,7 @@ class _KeywordVisitor extends GeneralizingAstVisitor {
   static bool _isPreviousTokenSynthetic(Object entity, TokenType type) {
     if (entity is AstNode) {
       Token token = entity.beginToken;
-      Token previousToken = token.previous;
+      Token previousToken = entity.findPrevious(token);
       return previousToken.isSynthetic && previousToken.type == type;
     }
     return false;
