@@ -2,8 +2,6 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library services.completion.contributor.dart.library_prefix;
-
 import 'dart:async';
 
 import 'package:analysis_server/src/provisional/completion/dart/completion_dart.dart';
@@ -21,11 +19,13 @@ class LibraryPrefixContributor extends DartCompletionContributor {
   @override
   Future<List<CompletionSuggestion>> computeSuggestions(
       DartCompletionRequest request) async {
+    // TODO(brianwilkerson) Determine whether this await is necessary.
+    await null;
     if (!request.includeIdentifiers) {
       return EMPTY_LIST;
     }
 
-    List<ImportElement> imports = await request.resolveImports();
+    List<ImportElement> imports = request.libraryElement.imports;
     if (imports == null) {
       return EMPTY_LIST;
     }
@@ -36,8 +36,7 @@ class LibraryPrefixContributor extends DartCompletionContributor {
       if (completion != null && completion.length > 0) {
         LibraryElement libElem = element.importedLibrary;
         if (libElem != null) {
-          CompletionSuggestion suggestion = createSuggestion(
-              libElem, request.ideOptions,
+          CompletionSuggestion suggestion = createSuggestion(libElem,
               completion: completion,
               kind: CompletionSuggestionKind.IDENTIFIER);
           if (suggestion != null) {
