@@ -2,17 +2,14 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library services.refactoring;
-
 import 'dart:async';
 
-import 'package:analysis_server/plugin/protocol/protocol.dart'
-    show RefactoringMethodParameter, SourceChange;
 import 'package:analysis_server/src/services/correction/status.dart';
 import 'package:analysis_server/src/services/refactoring/convert_getter_to_method.dart';
 import 'package:analysis_server/src/services/refactoring/convert_method_to_getter.dart';
 import 'package:analysis_server/src/services/refactoring/extract_local.dart';
 import 'package:analysis_server/src/services/refactoring/extract_method.dart';
+import 'package:analysis_server/src/services/refactoring/extract_widget.dart';
 import 'package:analysis_server/src/services/refactoring/inline_local.dart';
 import 'package:analysis_server/src/services/refactoring/inline_method.dart';
 import 'package:analysis_server/src/services/refactoring/move_file.dart';
@@ -24,12 +21,16 @@ import 'package:analysis_server/src/services/refactoring/rename_library.dart';
 import 'package:analysis_server/src/services/refactoring/rename_local.dart';
 import 'package:analysis_server/src/services/refactoring/rename_unit_member.dart';
 import 'package:analysis_server/src/services/search/search_engine.dart';
+import 'package:analyzer/dart/analysis/results.dart';
+import 'package:analyzer/dart/analysis/session.dart';
 import 'package:analyzer/dart/ast/ast.dart';
 import 'package:analyzer/dart/element/element.dart';
 import 'package:analyzer/file_system/file_system.dart';
+import 'package:analyzer/src/dart/analysis/driver.dart';
 import 'package:analyzer/src/dart/element/ast_provider.dart';
-import 'package:analyzer/src/generated/engine.dart';
 import 'package:analyzer/src/generated/source.dart';
+import 'package:analyzer_plugin/protocol/protocol_common.dart'
+    show RefactoringMethodParameter, SourceChange;
 
 /**
  * [Refactoring] to convert getters into normal [MethodDeclaration]s.
@@ -68,11 +69,8 @@ abstract class ExtractLocalRefactoring implements Refactoring {
   /**
    * Returns a new [ExtractLocalRefactoring] instance.
    */
-  factory ExtractLocalRefactoring(
-      CompilationUnit unit, int selectionOffset, int selectionLength) {
-    return new ExtractLocalRefactoringImpl(
-        unit, selectionOffset, selectionLength);
-  }
+  factory ExtractLocalRefactoring(ResolveResult resolveResult,
+      int selectionOffset, int selectionLength) = ExtractLocalRefactoringImpl;
 
   /**
    * The lengths of the expressions that cover the specified selection,
@@ -126,10 +124,16 @@ abstract class ExtractLocalRefactoring implements Refactoring {
    *
    * It does not perform all the checks (such as checking for conflicts with any
    * existing names in any of the scopes containing the current name), as many
-   * of these checkes require search engine. Use [checkFinalConditions] for this
+   * of these checks require search engine. Use [checkFinalConditions] for this
    * level of checking.
    */
   RefactoringStatus checkName();
+
+  /**
+   * Return `true` if refactoring is available, possibly without checking all
+   * initial conditions.
+   */
+  bool isAvailable();
 }
 
 /**
@@ -139,10 +143,14 @@ abstract class ExtractMethodRefactoring implements Refactoring {
   /**
    * Returns a new [ExtractMethodRefactoring] instance.
    */
-  factory ExtractMethodRefactoring(SearchEngine searchEngine,
-      CompilationUnit unit, int selectionOffset, int selectionLength) {
+  factory ExtractMethodRefactoring(
+      SearchEngine searchEngine,
+      AstProvider astProvider,
+      CompilationUnit unit,
+      int selectionOffset,
+      int selectionLength) {
     return new ExtractMethodRefactoringImpl(
-        searchEngine, unit, selectionOffset, selectionLength);
+        searchEngine, astProvider, unit, selectionOffset, selectionLength);
   }
 
   /**
@@ -215,10 +223,53 @@ abstract class ExtractMethodRefactoring implements Refactoring {
    *
    * It does not perform all the checks (such as checking for conflicts with any
    * existing names in any of the scopes containing the current name), as many
-   * of these checkes require search engine. Use [checkFinalConditions] for this
+   * of these checks require search engine. Use [checkFinalConditions] for this
    * level of checking.
    */
   RefactoringStatus checkName();
+
+  /**
+   * Return `true` if refactoring is available, possibly without checking all
+   * initial conditions.
+   */
+  bool isAvailable();
+}
+
+/**
+ * [Refactoring] to extract a widget creation expression or a method returning
+ * a widget, into a new stateless or stateful widget.
+ */
+abstract class ExtractWidgetRefactoring implements Refactoring {
+  /**
+   * Returns a new [ExtractWidgetRefactoring] instance.
+   */
+  factory ExtractWidgetRefactoring(SearchEngine searchEngine,
+      AnalysisSession session, CompilationUnit unit, int offset, int length) {
+    return new ExtractWidgetRefactoringImpl(
+        searchEngine, session, unit, offset, length);
+  }
+
+  /**
+   * The name that the class should be given.
+   */
+  void set name(String name);
+
+  /**
+   * Validates that the [name] is a valid identifier and is appropriate for a
+   * class.
+   *
+   * It does not perform all the checks (such as checking for conflicts with any
+   * existing names in any of the scopes containing the current name), as many
+   * of these checks require search engine. Use [checkFinalConditions] for this
+   * level of checking.
+   */
+  RefactoringStatus checkName();
+
+  /**
+   * Return `true` if refactoring is available, possibly without checking all
+   * initial conditions.
+   */
+  bool isAvailable();
 }
 
 /**
@@ -295,14 +346,10 @@ abstract class MoveFileRefactoring implements Refactoring {
   /**
    * Returns a new [MoveFileRefactoring] instance.
    */
-  factory MoveFileRefactoring(
-      ResourceProvider resourceProvider,
-      SearchEngine searchEngine,
-      AnalysisContext context,
-      Source source,
-      String oldFile) {
+  factory MoveFileRefactoring(ResourceProvider resourceProvider,
+      RefactoringWorkspace workspace, Source source, String oldFile) {
     return new MoveFileRefactoringImpl(
-        resourceProvider, searchEngine, context, source, oldFile);
+        resourceProvider, workspace, source, oldFile);
   }
 
   /**
@@ -363,6 +410,34 @@ abstract class Refactoring {
   bool requiresPreview();
 }
 
+/**
+ * Information about the workspace refactorings operate it.
+ */
+class RefactoringWorkspace {
+  final Iterable<AnalysisDriver> drivers;
+  final SearchEngine searchEngine;
+
+  RefactoringWorkspace(this.drivers, this.searchEngine);
+
+  /**
+   * Whether the file with the given [path] is in a context root.
+   */
+  bool containsFile(String path) {
+    return drivers.any((driver) {
+      return driver.contextRoot.containsFile(path);
+    });
+  }
+
+  /**
+   * Returns the drivers that have [path] in a context root.
+   */
+  Iterable<AnalysisDriver> driversContaining(String path) {
+    return drivers.where((driver) {
+      return driver.contextRoot.containsFile(path);
+    });
+  }
+}
+
 /**
  * Abstract [Refactoring] for renaming some [Element].
  */
@@ -372,8 +447,8 @@ abstract class RenameRefactoring implements Refactoring {
    * maybe `null` if there is no support for renaming [Element]s of the given
    * type.
    */
-  factory RenameRefactoring(
-      SearchEngine searchEngine, AstProvider astProvider, Element element) {
+  factory RenameRefactoring(RefactoringWorkspace workspace,
+      AstProvider astProvider, Element element) {
     if (element == null) {
       return null;
     }
@@ -381,26 +456,27 @@ abstract class RenameRefactoring implements Refactoring {
       element = (element as PropertyAccessorElement).variable;
     }
     if (element.enclosingElement is CompilationUnitElement) {
-      return new RenameUnitMemberRefactoringImpl(searchEngine, element);
+      return new RenameUnitMemberRefactoringImpl(workspace, element);
     }
     if (element is ConstructorElement) {
       return new RenameConstructorRefactoringImpl(
-          searchEngine, astProvider, element);
+          workspace, astProvider, element);
     }
     if (element is ImportElement) {
-      return new RenameImportRefactoringImpl(searchEngine, element);
+      return new RenameImportRefactoringImpl(workspace, astProvider, element);
     }
     if (element is LabelElement) {
-      return new RenameLabelRefactoringImpl(searchEngine, element);
+      return new RenameLabelRefactoringImpl(workspace, element);
     }
     if (element is LibraryElement) {
-      return new RenameLibraryRefactoringImpl(searchEngine, element);
+      return new RenameLibraryRefactoringImpl(workspace, element);
     }
     if (element is LocalElement) {
-      return new RenameLocalRefactoringImpl(searchEngine, astProvider, element);
+      return new RenameLocalRefactoringImpl(workspace, astProvider, element);
     }
     if (element.enclosingElement is ClassElement) {
-      return new RenameClassMemberRefactoringImpl(searchEngine, element);
+      return new RenameClassMemberRefactoringImpl(
+          workspace, astProvider, element);
     }
     return null;
   }
@@ -427,7 +503,7 @@ abstract class RenameRefactoring implements Refactoring {
    *
    * It does not perform all the checks (such as checking for conflicts with any
    * existing names in any of the scopes containing the current name), as many
-   * of these checkes require search engine. Use [checkFinalConditions] for this
+   * of these checks require search engine. Use [checkFinalConditions] for this
    * level of checking.
    */
   RefactoringStatus checkNewName();
@@ -451,9 +527,9 @@ class ResolvedUnitCache {
   }
 
   Future<CompilationUnit> getUnit(Element element) async {
-    CompilationUnitElement unitElement =
-        element.getAncestor((e) => e is CompilationUnitElement)
-            as CompilationUnitElement;
+    // TODO(brianwilkerson) Determine whether this await is necessary.
+    await null;
+    CompilationUnitElement unitElement = getUnitElement(element);
     CompilationUnit unit = _map[unitElement];
     if (unit == null) {
       unit = await _astProvider.getResolvedUnitForElement(element);
@@ -461,4 +537,9 @@ class ResolvedUnitCache {
     }
     return unit;
   }
+
+  CompilationUnitElement getUnitElement(Element element) {
+    return element.getAncestor((e) => e is CompilationUnitElement)
+        as CompilationUnitElement;
+  }
 }
