@@ -2,20 +2,16 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library socket.server;
-
-import 'package:analysis_server/plugin/protocol/protocol.dart';
+import 'package:analysis_server/protocol/protocol.dart';
+import 'package:analysis_server/protocol/protocol_generated.dart';
 import 'package:analysis_server/src/analysis_server.dart';
 import 'package:analysis_server/src/channel/channel.dart';
-import 'package:analysis_server/src/plugin/server_plugin.dart';
 import 'package:analysis_server/src/server/diagnostic_server.dart';
-import 'package:analysis_server/src/services/index/index.dart';
 import 'package:analyzer/file_system/physical_file_system.dart';
 import 'package:analyzer/instrumentation/instrumentation.dart';
-import 'package:analyzer/plugin/resolver_provider.dart';
-import 'package:analyzer/source/pub_package_map_provider.dart';
 import 'package:analyzer/src/generated/sdk.dart';
-import 'package:plugin/plugin.dart';
+import 'package:analyzer/src/plugin/resolver_provider.dart';
+import 'package:analyzer/src/source/pub_package_map_provider.dart';
 
 /**
  * Instances of the class [SocketServer] implement the common parts of
@@ -34,10 +30,8 @@ class SocketServer {
   final DartSdk defaultSdk;
   final InstrumentationService instrumentationService;
   final DiagnosticServer diagnosticServer;
-  final ServerPlugin serverPlugin;
   final ResolverProvider fileResolverProvider;
   final ResolverProvider packageResolverProvider;
-  final bool useSingleContextManager;
 
   /**
    * The analysis server that was created when a client established a
@@ -45,21 +39,14 @@ class SocketServer {
    */
   AnalysisServer analysisServer;
 
-  /**
-   * The plugins that are defined outside the analysis_server package.
-   */
-  List<Plugin> userDefinedPlugins;
-
   SocketServer(
       this.analysisServerOptions,
       this.sdkManager,
       this.defaultSdk,
       this.instrumentationService,
       this.diagnosticServer,
-      this.serverPlugin,
       this.fileResolverProvider,
-      this.packageResolverProvider,
-      this.useSingleContextManager);
+      this.packageResolverProvider);
 
   /**
    * Create an analysis server which will communicate with the client using the
@@ -78,34 +65,26 @@ class SocketServer {
 
     PhysicalResourceProvider resourceProvider;
     if (analysisServerOptions.fileReadMode == 'as-is') {
-      resourceProvider = PhysicalResourceProvider.INSTANCE;
+      resourceProvider = new PhysicalResourceProvider(null,
+          stateLocation: analysisServerOptions.cacheFolder);
     } else if (analysisServerOptions.fileReadMode == 'normalize-eol-always') {
       resourceProvider = new PhysicalResourceProvider(
-          PhysicalResourceProvider.NORMALIZE_EOL_ALWAYS);
+          PhysicalResourceProvider.NORMALIZE_EOL_ALWAYS,
+          stateLocation: analysisServerOptions.cacheFolder);
     } else {
       throw new Exception(
           'File read mode was set to the unknown mode: $analysisServerOptions.fileReadMode');
     }
 
-    Index index = null;
-    if (!analysisServerOptions.noIndex) {
-      index = createMemoryIndex();
-    }
-
     analysisServer = new AnalysisServer(
         serverChannel,
         resourceProvider,
         new PubPackageMapProvider(resourceProvider, defaultSdk),
-        index,
-        serverPlugin,
         analysisServerOptions,
         sdkManager,
         instrumentationService,
         diagnosticServer: diagnosticServer,
         fileResolverProvider: fileResolverProvider,
-        packageResolverProvider: packageResolverProvider,
-        useSingleContextManager: useSingleContextManager,
-        rethrowExceptions: false);
-    analysisServer.userDefinedPlugins = userDefinedPlugins;
+        packageResolverProvider: packageResolverProvider);
   }
 }
