@@ -2,13 +2,12 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library codegen.protocol;
-
 import 'dart:convert';
 
 import 'package:analyzer/src/codegen/tools.dart';
 import 'package:front_end/src/codegen/tools.dart';
 import 'package:html/dom.dart' as dom;
+import 'package:path/path.dart' as path;
 
 import 'api.dart';
 import 'codegen_dart.dart';
@@ -29,11 +28,14 @@ const Map<String, String> specialElementFlags = const {
   'deprecated': '0x20'
 };
 
-final GeneratedFile target = new GeneratedFile(
-    'lib/plugin/protocol/generated_protocol.dart', (String pkgPath) {
-  CodegenProtocolVisitor visitor = new CodegenProtocolVisitor(readApi(pkgPath));
-  return visitor.collectCode(visitor.visitApi);
-});
+GeneratedFile target(bool responseRequiresRequestTime) {
+  return new GeneratedFile('lib/protocol/protocol_generated.dart',
+      (String pkgPath) async {
+    CodegenProtocolVisitor visitor = new CodegenProtocolVisitor(
+        path.basename(pkgPath), responseRequiresRequestTime, readApi(pkgPath));
+    return visitor.collectCode(visitor.visitApi);
+  });
+}
 
 /**
  * Callback type used to represent arbitrary code generation.
@@ -67,6 +69,17 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
   static const String disclaimer =
       'Clients may not extend, implement or mix-in this class.';
 
+  /**
+   * The name of the package into which code is being generated.
+   */
+  final String packageName;
+
+  /**
+   * A flag indicating whether the class [Response] requires a `requestTime`
+   * parameter.
+   */
+  final bool responseRequiresRequestTime;
+
   /**
    * Visitor used to produce doc comments.
    */
@@ -79,7 +92,8 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
    */
   final Map<String, ImpliedType> impliedTypes;
 
-  CodegenProtocolVisitor(Api api)
+  CodegenProtocolVisitor(
+      this.packageName, this.responseRequiresRequestTime, Api api)
       : toHtmlVisitor = new ToHtmlVisitor(api),
         impliedTypes = computeImpliedTypes(api),
         super(api) {
@@ -114,13 +128,14 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
   }
 
   /**
-   * Translate each type implied by the API to a class.
+   * Translate each of the given [types] implied by the API to a class.
    */
-  void emitClasses() {
-    for (ImpliedType impliedType in impliedTypes.values) {
+  void emitClasses(List<ImpliedType> types) {
+    for (ImpliedType impliedType in types) {
       TypeDecl type = impliedType.type;
       String dartTypeName = capitalize(impliedType.camelName);
       if (type == null) {
+        writeln();
         emitEmptyObjectClass(dartTypeName, impliedType);
       } else if (type is TypeObject) {
         writeln();
@@ -153,14 +168,14 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
       case 'requestParams':
         inputType = 'Request';
         inputName = 'request';
-        fieldName = '_params';
+        fieldName = 'params';
         makeDecoder = 'new RequestDecoder(request)';
         constructorName = 'fromRequest';
         break;
       case 'requestResult':
         inputType = 'Response';
         inputName = 'response';
-        fieldName = '_result';
+        fieldName = 'result';
         makeDecoder =
             'new ResponseDecoder(REQUEST_ID_REFACTORING_KINDS.remove(response.id))';
         constructorName = 'fromResponse';
@@ -168,7 +183,7 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
       case 'notificationParams':
         inputType = 'Notification';
         inputName = 'notification';
-        fieldName = '_params';
+        fieldName = 'params';
         makeDecoder = 'new ResponseDecoder(null)';
         constructorName = 'fromNotification';
         break;
@@ -219,13 +234,22 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
     }));
     write('class $className');
     if (impliedType.kind == 'refactoringFeedback') {
-      writeln(' extends RefactoringFeedback {');
+      writeln(' extends RefactoringFeedback implements HasToJson {');
     } else if (impliedType.kind == 'refactoringOptions') {
-      writeln(' extends RefactoringOptions {');
+      writeln(' extends RefactoringOptions implements HasToJson {');
+    } else if (impliedType.kind == 'requestParams') {
+      writeln(' implements RequestParams {');
+    } else if (impliedType.kind == 'requestResult') {
+      writeln(' implements ResponseResult {');
     } else {
       writeln(' {');
     }
     indent(() {
+      if (impliedType.kind == 'requestResult' ||
+          impliedType.kind == 'requestParams') {
+        emitEmptyToJsonMember();
+        writeln();
+      }
       if (emitToRequestMember(impliedType)) {
         writeln();
       }
@@ -242,6 +266,14 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
     writeln('}');
   }
 
+  /**
+   * Emit the toJson() code for an empty class.
+   */
+  void emitEmptyToJsonMember() {
+    writeln('@override');
+    writeln('Map<String, dynamic> toJson() => <String, dynamic>{};');
+  }
+
   /**
    * Emit a class to encapsulate an enum.
    */
@@ -292,6 +324,7 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
       writeln('];');
       writeln();
 
+      writeln('@override');
       writeln('final String name;');
       writeln();
       writeln('const $className._(this.name);');
@@ -368,6 +401,20 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
     writeln('}');
   }
 
+  void emitImports() {
+    writeln("import 'dart:convert' hide JsonDecoder;");
+    writeln();
+    writeln("import 'package:analyzer/src/generated/utilities_general.dart';");
+    writeln("import 'package:$packageName/protocol/protocol.dart';");
+    writeln(
+        "import 'package:$packageName/src/protocol/protocol_internal.dart';");
+    for (String uri in api.types.importUris) {
+      write("import '");
+      write(uri);
+      writeln("';");
+    }
+  }
+
   /**
    * Emit the class to encapsulate an object type.
    */
@@ -389,6 +436,10 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
       writeln(' extends RefactoringFeedback {');
     } else if (impliedType.kind == 'refactoringOptions') {
       writeln(' extends RefactoringOptions {');
+    } else if (impliedType.kind == 'requestParams') {
+      writeln(' implements RequestParams {');
+    } else if (impliedType.kind == 'requestResult') {
+      writeln(' implements ResponseResult {');
     } else {
       writeln(' implements HasToJson {');
     }
@@ -453,7 +504,7 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
         writeln();
       }
       writeln('@override');
-      writeln('String toString() => JSON.encode(toJson());');
+      writeln('String toString() => json.encode(toJson());');
       writeln();
       emitObjectEqualsMember(type, className);
       writeln();
@@ -631,7 +682,7 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
             writeln(' else {');
             indent(() {
               writeln(
-                  "throw jsonDecoder.missingKey(jsonPath, $fieldNameString);");
+                  "throw jsonDecoder.mismatch(jsonPath, $fieldNameString);");
             });
             writeln('}');
           } else {
@@ -840,6 +891,7 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
    * Emit the toJson() code for an object class.
    */
   void emitToJsonMember(TypeObject type) {
+    writeln('@override');
     writeln('Map<String, dynamic> toJson() {');
     indent(() {
       writeln('Map<String, dynamic> result = {};');
@@ -891,6 +943,7 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
    */
   bool emitToRequestMember(ImpliedType impliedType) {
     if (impliedType.kind == 'requestParams') {
+      writeln('@override');
       writeln('Request toRequest(String id) {');
       indent(() {
         String methodString =
@@ -910,10 +963,19 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
    */
   bool emitToResponseMember(ImpliedType impliedType) {
     if (impliedType.kind == 'requestResult') {
-      writeln('Response toResponse(String id) {');
+      writeln('@override');
+      if (responseRequiresRequestTime) {
+        writeln('Response toResponse(String id, int requestTime) {');
+      } else {
+        writeln('Response toResponse(String id) {');
+      }
       indent(() {
         String jsonPart = impliedType.type != null ? 'toJson()' : 'null';
-        writeln('return new Response(id, result: $jsonPart);');
+        if (responseRequiresRequestTime) {
+          writeln('return new Response(id, requestTime, result: $jsonPart);');
+        } else {
+          writeln('return new Response(id, result: $jsonPart);');
+        }
       });
       writeln('}');
       return true;
@@ -1017,6 +1079,19 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
     }
   }
 
+  /**
+   * Return a list of the classes to be emitted.
+   */
+  List<ImpliedType> getClassesToEmit() {
+    List<ImpliedType> types = impliedTypes.values.where((ImpliedType type) {
+      ApiNode node = type.apiNode;
+      return !(node is TypeDefinition && node.isExternal);
+    }).toList();
+    types.sort((first, second) =>
+        capitalize(first.camelName).compareTo(capitalize(second.camelName)));
+    return types;
+  }
+
   /**
    * True if the constructor argument for the given field should be optional.
    */
@@ -1035,7 +1110,7 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
    * Create a string literal that evaluates to [s].
    */
   String literalString(String s) {
-    return JSON.encode(s);
+    return json.encode(s);
   }
 
   /**
@@ -1095,11 +1170,10 @@ class CodegenProtocolVisitor extends DartCodegenVisitor with CodeGenerator {
 
   @override
   visitApi() {
-    outputHeader();
+    outputHeader(year: '2017');
     writeln();
-    writeln('part of analysis_server.plugin.protocol.protocol;');
-    writeln();
-    emitClasses();
+    emitImports();
+    emitClasses(getClassesToEmit());
   }
 }
 
@@ -1130,6 +1204,7 @@ abstract class FromJsonCode {
  * Representation of FromJsonCode for a function defined elsewhere.
  */
 class FromJsonFunction extends FromJsonCode {
+  @override
   final String asClosure;
 
   FromJsonFunction(this.asClosure);
@@ -1201,6 +1276,7 @@ abstract class ToJsonCode {
  * Representation of ToJsonCode for a function defined elsewhere.
  */
 class ToJsonFunction extends ToJsonCode {
+  @override
   final String asClosure;
 
   ToJsonFunction(this.asClosure);
