@@ -5,8 +5,6 @@
 /**
  * Tools for Java code generation.
  */
-library CodegenJava;
-
 import 'package:analyzer/src/codegen/tools.dart';
 import 'package:front_end/src/codegen/tools.dart';
 import 'package:html/dom.dart' as dom;
@@ -21,7 +19,7 @@ import 'to_html.dart';
  */
 GeneratedFile javaGeneratedFile(
     String path, CodegenJavaVisitor createVisitor(Api api)) {
-  return new GeneratedFile(path, (String pkgPath) {
+  return new GeneratedFile(path, (String pkgPath) async {
     CodegenJavaVisitor visitor = createVisitor(readApi(pkgPath));
     return visitor.collectCode(visitor.visitApi);
   });
