@@ -11,11 +11,27 @@ import 'dart:io' as io;
 import 'package:analyzer/file_system/file_system.dart';
 import 'package:analyzer/src/generated/source_io.dart';
 import 'package:analyzer/src/source/source_resource.dart';
-import 'package:analyzer/src/util/absolute_path.dart';
-import 'package:isolate/isolate_runner.dart';
 import 'package:path/path.dart';
 import 'package:watcher/watcher.dart';
 
+/**
+ * The name of the directory containing plugin specific subfolders used to
+ * store data across sessions.
+ */
+const String _SERVER_DIR = ".dartServer";
+
+/**
+ * Returns the path to the user's home directory.
+ */
+String _getStandardStateLocation() {
+  final home = io.Platform.isWindows
+      ? io.Platform.environment['LOCALAPPDATA']
+      : io.Platform.environment['HOME'];
+  return home != null && io.FileSystemEntity.isDirectorySync(home)
+      ? join(home, _SERVER_DIR)
+      : null;
+}
+
 /**
  * Return modification times for every file path in [paths].
  *
@@ -43,33 +59,27 @@ List<int> _pathsToTimes(List<String> paths) {
  * A `dart:io` based implementation of [ResourceProvider].
  */
 class PhysicalResourceProvider implements ResourceProvider {
-  static final FileReadMode NORMALIZE_EOL_ALWAYS =
+  static final String Function(String) NORMALIZE_EOL_ALWAYS =
       (String string) => string.replaceAll(new RegExp('\r\n?'), '\n');
 
   static final PhysicalResourceProvider INSTANCE =
       new PhysicalResourceProvider(null);
 
   /**
-   * The name of the directory containing plugin specific subfolders used to
-   * store data across sessions.
+   * The path to the base folder where state is stored.
    */
-  static final String SERVER_DIR = ".dartServer";
+  final String _stateLocation;
 
-  static _SingleIsolateRunnerProvider pathsToTimesIsolateProvider =
-      new _SingleIsolateRunnerProvider();
-
-  @override
-  final AbsolutePathContext absolutePathContext =
-      new AbsolutePathContext(io.Platform.isWindows);
-
-  PhysicalResourceProvider(FileReadMode fileReadMode) {
+  PhysicalResourceProvider(String Function(String) fileReadMode,
+      {String stateLocation})
+      : _stateLocation = stateLocation ?? _getStandardStateLocation() {
     if (fileReadMode != null) {
       FileBasedSource.fileReadMode = fileReadMode;
     }
   }
 
   @override
-  Context get pathContext => io.Platform.isWindows ? windows : posix;
+  Context get pathContext => context;
 
   @override
   File getFile(String path) {
@@ -85,9 +95,10 @@ class PhysicalResourceProvider implements ResourceProvider {
 
   @override
   Future<List<int>> getModificationTimes(List<Source> sources) async {
+    // TODO(brianwilkerson) Determine whether this await is necessary.
+    await null;
     List<String> paths = sources.map((source) => source.fullName).toList();
-    IsolateRunner runner = await pathsToTimesIsolateProvider.get();
-    return runner.run(_pathsToTimes, paths);
+    return _pathsToTimes(paths);
   }
 
   @override
@@ -101,15 +112,8 @@ class PhysicalResourceProvider implements ResourceProvider {
 
   @override
   Folder getStateLocation(String pluginId) {
-    String home;
-    if (io.Platform.isWindows) {
-      home = io.Platform.environment['LOCALAPPDATA'];
-    } else {
-      home = io.Platform.environment['HOME'];
-    }
-    if (home != null && io.FileSystemEntity.isDirectorySync(home)) {
-      io.Directory directory =
-          new io.Directory(join(home, SERVER_DIR, pluginId));
+    if (_stateLocation != null) {
+      io.Directory directory = new io.Directory(join(_stateLocation, pluginId));
       directory.createSync(recursive: true);
       return new _PhysicalFolder(directory);
     }
@@ -149,6 +153,14 @@ class _PhysicalFile extends _PhysicalResource implements File {
    */
   io.File get _file => _entry as io.File;
 
+  @override
+  File copyTo(Folder parentFolder) {
+    parentFolder.create();
+    File destination = parentFolder.getChildAssumingFile(shortName);
+    destination.writeAsBytesSync(readAsBytesSync());
+    return destination;
+  }
+
   @override
   Source createSource([Uri uri]) {
     return new FileSource(this, uri ?? pathContext.toUri(path));
@@ -226,7 +238,9 @@ class _PhysicalFolder extends _PhysicalResource implements Folder {
   _PhysicalFolder(io.Directory directory) : super(directory);
 
   @override
-  Stream<WatchEvent> get changes => new DirectoryWatcher(_entry.path).events;
+  Stream<WatchEvent> get changes =>
+      new DirectoryWatcher(_entry.path).events.handleError((error) {},
+          test: (error) => error is io.FileSystemException);
 
   /**
    * Return the underlying file being represented by this wrapper.
@@ -240,7 +254,22 @@ class _PhysicalFolder extends _PhysicalResource implements Folder {
 
   @override
   bool contains(String path) {
-    return absolutePathContext.isWithin(this.path, path);
+    return pathContext.isWithin(this.path, path);
+  }
+
+  @override
+  Folder copyTo(Folder parentFolder) {
+    Folder destination = parentFolder.getChildAssumingFolder(shortName);
+    destination.create();
+    for (Resource child in getChildren()) {
+      child.copyTo(destination);
+    }
+    return destination;
+  }
+
+  @override
+  void create() {
+    _directory.createSync(recursive: true);
   }
 
   @override
@@ -314,9 +343,6 @@ abstract class _PhysicalResource implements Resource {
 
   _PhysicalResource(this._entry);
 
-  AbsolutePathContext get absolutePathContext =>
-      PhysicalResourceProvider.INSTANCE.absolutePathContext;
-
   @override
   bool get exists => _entry.existsSync();
 
@@ -325,7 +351,7 @@ abstract class _PhysicalResource implements Resource {
 
   @override
   Folder get parent {
-    String parentPath = absolutePathContext.dirname(path);
+    String parentPath = pathContext.dirname(path);
     if (parentPath == path) {
       return null;
     }
@@ -341,7 +367,7 @@ abstract class _PhysicalResource implements Resource {
   Context get pathContext => io.Platform.isWindows ? windows : posix;
 
   @override
-  String get shortName => absolutePathContext.basename(path);
+  String get shortName => pathContext.basename(path);
 
   @override
   bool operator ==(other) {
@@ -390,33 +416,3 @@ abstract class _PhysicalResource implements Resource {
     }
   }
 }
-
-/**
- * This class encapsulates logic for creating a single [IsolateRunner].
- */
-class _SingleIsolateRunnerProvider {
-  bool _isSpawning = false;
-  IsolateRunner _runner;
-
-  /**
-   * Complete with the only [IsolateRunner] instance.
-   */
-  Future<IsolateRunner> get() async {
-    if (_runner != null) {
-      return _runner;
-    }
-    if (_isSpawning) {
-      Completer<IsolateRunner> completer = new Completer<IsolateRunner>();
-      new Timer.periodic(new Duration(milliseconds: 10), (Timer timer) {
-        if (_runner != null) {
-          completer.complete(_runner);
-          timer.cancel();
-        }
-      });
-      return completer.future;
-    }
-    _isSpawning = true;
-    _runner = await IsolateRunner.spawn();
-    return _runner;
-  }
-}
