@@ -13,13 +13,20 @@ import 'package:analyzer/instrumentation/instrumentation.dart';
  * An [InstrumentationServer] that writes to a file.
  */
 class FileInstrumentationServer implements InstrumentationServer {
+  final String filePath;
   IOSink _sink;
 
-  FileInstrumentationServer(String path) {
-    File file = new File(path);
+  FileInstrumentationServer(this.filePath) {
+    File file = new File(filePath);
     _sink = file.openWrite();
   }
 
+  @override
+  String get describe => "file: $filePath";
+
+  @override
+  String get sessionId => '';
+
   @override
   void log(String message) {
     _sink.writeln(message);
@@ -32,6 +39,8 @@ class FileInstrumentationServer implements InstrumentationServer {
 
   @override
   Future shutdown() async {
+    // TODO(brianwilkerson) Determine whether this await is necessary.
+    await null;
     await _sink.close();
     _sink = null;
   }
