@@ -14,12 +14,19 @@ import 'package:analyzer/error/error.dart';
  * might aggregated to define a project's style guide.
  */
 class LintCode extends ErrorCode {
-  const LintCode(String name, String message, [String correction])
-      : super(name, message, correction);
+  const LintCode(String name, String message, {String correction})
+      : super.temporary(name, message, correction: correction);
 
   @override
   ErrorSeverity get errorSeverity => ErrorSeverity.INFO;
 
   @override
   ErrorType get type => ErrorType.LINT;
+
+  /**
+   * Overridden so that [LintCode] and its subclasses share the same uniqueName
+   * pattern (we know how to identify a lint even if we don't know the specific
+   * subclass the lint's code is defined in.
+   */
+  String get uniqueName => "LintCode.$name";
 }
