@@ -57,6 +57,56 @@ import 'format.dart' as generated;
  */
 const informative = null;
 
+/**
+ * Information about the context of an exception in analysis driver.
+ */
+@TopLevel('ADEC')
+abstract class AnalysisDriverExceptionContext extends base.SummaryClass {
+  factory AnalysisDriverExceptionContext.fromBuffer(List<int> buffer) =>
+      generated.readAnalysisDriverExceptionContext(buffer);
+
+  /**
+   * The exception string.
+   */
+  @Id(1)
+  String get exception;
+
+  /**
+   * The state of files when the exception happened.
+   */
+  @Id(3)
+  List<AnalysisDriverExceptionFile> get files;
+
+  /**
+   * The path of the file being analyzed when the exception happened.
+   */
+  @Id(0)
+  String get path;
+
+  /**
+   * The exception stack trace string.
+   */
+  @Id(2)
+  String get stackTrace;
+}
+
+/**
+ * Information about a single file in [AnalysisDriverExceptionContext].
+ */
+abstract class AnalysisDriverExceptionFile extends base.SummaryClass {
+  /**
+   * The content of the file.
+   */
+  @Id(1)
+  String get content;
+
+  /**
+   * The path of the file.
+   */
+  @Id(0)
+  String get path;
+}
+
 /**
  * Information about a resolved unit.
  */
@@ -78,6 +128,31 @@ abstract class AnalysisDriverResolvedUnit extends base.SummaryClass {
   AnalysisDriverUnitIndex get index;
 }
 
+/**
+ * Information about a subtype of one or more classes.
+ */
+abstract class AnalysisDriverSubtype extends base.SummaryClass {
+  /**
+   * The names of defined instance members.
+   * The list is sorted in ascending order.
+   */
+  @Id(2)
+  List<String> get members;
+
+  /**
+   * The name of the class.
+   */
+  @Id(0)
+  String get name;
+
+  /**
+   * The identifiers of the direct supertypes.
+   * The list is sorted in ascending order.
+   */
+  @Id(1)
+  List<String> get supertypes;
+}
+
 /**
  * Information about an error in a resolved unit.
  */
@@ -177,6 +252,12 @@ abstract class AnalysisDriverUnitIndex extends base.SummaryClass {
   @Id(0)
   List<String> get strings;
 
+  /**
+   * The list of classes declared in the unit.
+   */
+  @Id(18)
+  List<AnalysisDriverSubtype> get subtypes;
+
   /**
    * Each item of this list corresponds to the library URI of a unique library
    * specific unit referenced in the index.  It is an index into [strings] list.
@@ -263,12 +344,31 @@ abstract class AnalysisDriverUnlinkedUnit extends base.SummaryClass {
   factory AnalysisDriverUnlinkedUnit.fromBuffer(List<int> buffer) =>
       generated.readAnalysisDriverUnlinkedUnit(buffer);
 
+  /**
+   * List of class member names defined by the unit.
+   */
+  @Id(3)
+  List<String> get definedClassMemberNames;
+
+  /**
+   * List of top-level names defined by the unit.
+   */
+  @Id(2)
+  List<String> get definedTopLevelNames;
+
   /**
    * List of external names referenced by the unit.
    */
   @Id(0)
   List<String> get referencedNames;
 
+  /**
+   * List of names which are used in `extends`, `with` or `implements` clauses
+   * in the file. Import prefixes and type arguments are not included.
+   */
+  @Id(4)
+  List<String> get subtypedNames;
+
   /**
    * Unlinked information for the unit.
    */
@@ -294,11 +394,23 @@ abstract class CodeRange extends base.SummaryClass {
 }
 
 /**
- * Summary information about a reference to a an entity such as a type, top
- * level executable, or executable within a class.
+ * Summary information about a reference to an entity such as a type, top level
+ * executable, or executable within a class.
  */
 abstract class EntityRef extends base.SummaryClass {
   /**
+   * The kind of entity being represented.
+   */
+  @Id(8)
+  EntityRefKind get entityKind;
+
+  /**
+   * Notice: This will be deprecated. However, its not deprecated yet, as we're
+   * keeping it for backwards compatibilty, and marking it deprecated makes it
+   * unreadable.
+   *
+   * TODO(mfairhurst) mark this deprecated, and remove its logic.
+   *
    * If this is a reference to a function type implicitly defined by a
    * function-typed parameter, a list of zero-based indices indicating the path
    * from the entity referred to by [reference] to the appropriate type
@@ -352,6 +464,20 @@ abstract class EntityRef extends base.SummaryClass {
   @Id(0)
   int get reference;
 
+  /**
+   * If this [EntityRef] appears in a syntactic context where its type arguments
+   * might need to be inferred by a method other than instantiate-to-bounds,
+   * and [typeArguments] is empty, a slot id (which is unique within the
+   * compilation unit).  If an entry appears in [LinkedUnit.types] whose [slot]
+   * matches this value, that entry will contain the complete inferred type.
+   *
+   * This is called `refinedSlot` to clarify that if it points to an inferred
+   * type, it points to a type that is a "refinement" of this one (one in which
+   * some type arguments have been inferred).
+   */
+  @Id(9)
+  int get refinedSlot;
+
   /**
    * If this [EntityRef] is contained within [LinkedUnit.types], slot id (which
    * is unique within the compilation unit) identifying the target of type
@@ -386,6 +512,34 @@ abstract class EntityRef extends base.SummaryClass {
    */
   @Id(1)
   List<EntityRef> get typeArguments;
+
+  /**
+   * If this is a function type, the type parameters defined for the function
+   * type (if any).
+   */
+  @Id(7)
+  List<UnlinkedTypeParam> get typeParameters;
+}
+
+/**
+ * Enum used to indicate the kind of an entity reference.
+ */
+enum EntityRefKind {
+  /**
+   * The entity represents a named type.
+   */
+  named,
+
+  /**
+   * The entity represents a generic function type.
+   */
+  genericFunctionType,
+
+  /**
+   * The entity represents a function type that was synthesized by a LUB
+   * computation.
+   */
+  syntheticFunction
 }
 
 /**
@@ -534,18 +688,14 @@ enum IndexSyntheticElementKind {
  */
 abstract class LinkedDependency extends base.SummaryClass {
   /**
-   * URI for the compilation units listed in the library's `part` declarations.
-   * These URIs are relative to the importing library.
+   * Absolute URI for the compilation units listed in the library's `part`
+   * declarations, empty string for invalid URI.
    */
   @Id(1)
   List<String> get parts;
 
   /**
-   * The relative URI of the dependent library.  This URI is relative to the
-   * importing library, even if there are intervening `export` declarations.
-   * So, for example, if `a.dart` imports `b/c.dart` and `b/c.dart` exports
-   * `d/e.dart`, the URI listed for `a.dart`'s dependency on `e.dart` will be
-   * `b/d/e.dart`.
+   * The absolute URI of the dependent library, e.g. `package:foo/bar.dart`.
    */
   @Id(0)
   String get uri;
@@ -701,10 +851,9 @@ abstract class LinkedReference extends base.SummaryClass {
   /**
    * If [kind] is [ReferenceKind.function] (that is, the entity being referred
    * to is a local function), the index of the function within
-   * [UnlinkedExecutable.localFunctions].  If [kind] is
-   * [ReferenceKind.variable], the index of the variable within
-   * [UnlinkedExecutable.localVariables].  Otherwise zero.
+   * [UnlinkedExecutable.localFunctions].  Otherwise zero.
    */
+  @deprecated
   @Id(6)
   int get localIndex;
 
@@ -749,9 +898,9 @@ abstract class LinkedUnit extends base.SummaryClass {
   List<int> get constCycles;
 
   /**
-   * List of slot ids (referring to [UnlinkedParam.inheritsCovariantSlot])
-   * corresponding to parameters that inherit `@covariant` behavior from a base
-   * class.
+   * List of slot ids (referring to [UnlinkedParam.inheritsCovariantSlot] or
+   * [UnlinkedVariable.inheritsCovariantSlot]) corresponding to parameters
+   * that inherit `@covariant` behavior from a base class.
    */
   @Id(3)
   List<int> get parametersInheritingCovariant;
@@ -767,6 +916,12 @@ abstract class LinkedUnit extends base.SummaryClass {
   @Id(0)
   List<LinkedReference> get references;
 
+  /**
+   * The list of type inference errors.
+   */
+  @Id(4)
+  List<TopLevelInferenceError> get topLevelInferenceErrors;
+
   /**
    * List associating slot ids found inside the unlinked summary for the
    * compilation unit with propagated and inferred types.
@@ -908,8 +1063,8 @@ abstract class PackageIndex extends base.SummaryClass {
 
   /**
    * Each item of this list corresponds to a unique referenced element.  It is
-   * the identifier of the class member element name, or `null` if the element is
-   * a top-level element.  The list is sorted in ascending order, so that the
+   * the identifier of the class member element name, or `null` if the element
+   * is a top-level element.  The list is sorted in ascending order, so that the
    * client can quickly check whether an element is referenced in this
    * [PackageIndex].
    */
@@ -1034,7 +1189,77 @@ enum ReferenceKind {
   /**
    * The entity being referred to does not exist.
    */
-  unresolved
+  unresolved,
+
+  /**
+   * The entity is a typedef expressed using generic function type syntax.
+   */
+  genericFunctionTypedef
+}
+
+/**
+ * Summary information about a top-level type inference error.
+ */
+abstract class TopLevelInferenceError extends base.SummaryClass {
+  /**
+   * The [kind] specific arguments.
+   */
+  @Id(2)
+  List<String> get arguments;
+
+  /**
+   * The kind of the error.
+   */
+  @Id(1)
+  TopLevelInferenceErrorKind get kind;
+
+  /**
+   * The slot id (which is unique within the compilation unit) identifying the
+   * target of type inference with which this [TopLevelInferenceError] is
+   * associated.
+   */
+  @Id(0)
+  int get slot;
+}
+
+/**
+ * Enum used to indicate the kind of the error during top-level inference.
+ */
+enum TopLevelInferenceErrorKind {
+  assignment,
+  instanceGetter,
+  dependencyCycle,
+  overrideConflictFieldType,
+  overrideConflictReturnType,
+  overrideConflictParameterType
+}
+
+/**
+ * Enum used to indicate the style of a typedef.
+ */
+enum TypedefStyle {
+  /**
+   * A typedef that defines a non-generic function type. The syntax is
+   * ```
+   * 'typedef' returnType? identifier typeParameters? formalParameterList ';'
+   * ```
+   * The typedef can have type parameters associated with it, but the function
+   * type that results from applying type arguments does not.
+   */
+  functionType,
+
+  /**
+   * A typedef expressed using generic function type syntax. The syntax is
+   * ```
+   * typeAlias ::=
+   *     'typedef' identifier typeParameters? '=' genericFunctionType ';'
+   * genericFunctionType ::=
+   *     returnType? 'Function' typeParameters? parameterTypeList
+   * ```
+   * Both the typedef itself and the function type that results from applying
+   * type arguments can have type parameters.
+   */
+  genericFunctionType
 }
 
 /**
@@ -1434,6 +1659,12 @@ abstract class UnlinkedEnum extends base.SummaryClass {
  * declaration.
  */
 abstract class UnlinkedEnumValue extends base.SummaryClass {
+  /**
+   * Annotations for this value.
+   */
+  @Id(3)
+  List<UnlinkedExpr> get annotations;
+
   /**
    * Documentation comment for the enum value, or `null` if there is no
    * documentation comment.
@@ -1589,13 +1820,15 @@ abstract class UnlinkedExecutable extends base.SummaryClass {
    * The list of local labels.
    */
   @informative
+  @deprecated
   @Id(22)
-  List<UnlinkedLabel> get localLabels;
+  List<String> get localLabels;
 
   /**
    * The list of local variables.
    */
   @informative
+  @deprecated
   @Id(19)
   List<UnlinkedVariable> get localVariables;
 
@@ -2034,6 +2267,11 @@ enum UnlinkedExprOperation {
    * from [UnlinkedExpr.references], and push the resulting value back onto the
    * stack.
    *
+   * Arguments are skipped, and `0` are specified as the numbers of arguments
+   * on the stack, if the expression is not a constant. We store expression of
+   * variable initializers to perform top-level inference, and arguments are
+   * never used to infer types.
+   *
    * Note that for an invocation of the form `const a.b(...)` (where no type
    * arguments are specified), it is impossible to tell from the unresolved AST
    * alone whether `a` is a class name and `b` is a constructor name, or `a` is
@@ -2069,8 +2307,8 @@ enum UnlinkedExprOperation {
   /**
    * Pop the top 2*n values from the stack (where n is obtained from
    * [UnlinkedExpr.ints]), interpret them as key/value pairs, place them in a
-   * [Map], and push the result back onto the stack.  The two type parameters for
-   * the [Map] are obtained from [UnlinkedExpr.references].
+   * [Map], and push the result back onto the stack.  The two type parameters
+   * for the [Map] are obtained from [UnlinkedExpr.references].
    */
   makeTypedMap,
 
@@ -2277,6 +2515,11 @@ enum UnlinkedExprOperation {
    * aforementioned method or function.  Push the result of the invocation onto
    * the stack.
    *
+   * Arguments are skipped, and `0` are specified as the numbers of arguments
+   * on the stack, if the expression is not a constant. We store expression of
+   * variable initializers to perform top-level inference, and arguments are
+   * never used to infer types.
+   *
    * In general `a.b` cannot not be distinguished between: `a` is a prefix and
    * `b` is a top-level function; or `a` is an object and `b` is the name of a
    * method.  This operation should be used for a sequence of identifiers
@@ -2298,6 +2541,11 @@ enum UnlinkedExprOperation {
    * arguments for the aforementioned method.  Push the result of the
    * invocation onto the stack.
    *
+   * Arguments are skipped, and `0` are specified as the numbers of arguments
+   * on the stack, if the expression is not a constant. We store expression of
+   * variable initializers to perform top-level inference, and arguments are
+   * never used to infer types.
+   *
    * This operation should be used for invocation of a method invocation
    * where `target` is known to be an object instance.
    */
@@ -2388,6 +2636,11 @@ enum UnlinkedExprOperation {
    * Push `this` expression onto the stack.
    */
   pushThis,
+
+  /**
+   * Push `super` expression onto the stack.
+   */
+  pushSuper,
 }
 
 /**
@@ -2473,37 +2726,6 @@ abstract class UnlinkedImport extends base.SummaryClass {
   int get uriOffset;
 }
 
-/**
- * Unlinked summary information about a label.
- */
-abstract class UnlinkedLabel extends base.SummaryClass {
-  /**
-   * Return `true` if this label is associated with a `switch` member (`case` or
-   * `default`).
-   */
-  @Id(2)
-  bool get isOnSwitchMember;
-
-  /**
-   * Return `true` if this label is associated with a `switch` statement.
-   */
-  @Id(3)
-  bool get isOnSwitchStatement;
-
-  /**
-   * Name of the label.
-   */
-  @Id(0)
-  String get name;
-
-  /**
-   * Offset of the label relative to the beginning of the file.
-   */
-  @informative
-  @Id(1)
-  int get nameOffset;
-}
-
 /**
  * Unlinked summary information about a function parameter.
  */
@@ -2555,8 +2777,8 @@ abstract class UnlinkedParam extends base.SummaryClass {
   int get inheritsCovariantSlot;
 
   /**
-   * The synthetic initializer function of the parameter.  Absent if the variable
-   * does not have an initializer.
+   * The synthetic initializer function of the parameter.  Absent if the
+   * variable does not have an initializer.
    */
   @Id(12)
   UnlinkedExecutable get initializer;
@@ -2574,7 +2796,14 @@ abstract class UnlinkedParam extends base.SummaryClass {
   bool get isFinal;
 
   /**
-   * Indicates whether this is a function-typed parameter.
+   * Indicates whether this is a function-typed parameter. A parameter is
+   * function-typed if the declaration of the parameter has explicit formal
+   * parameters
+   * ```
+   * int functionTyped(int p)
+   * ```
+   * but is not function-typed if it does not, even if the type of the parameter
+   * is a function type.
    */
   @Id(5)
   bool get isFunctionTyped;
@@ -2823,11 +3052,19 @@ abstract class UnlinkedTypedef extends base.SummaryClass {
   List<UnlinkedParam> get parameters;
 
   /**
-   * Return type of the typedef.
+   * If [style] is [TypedefStyle.functionType], the return type of the typedef.
+   * If [style] is [TypedefStyle.genericFunctionType], the function type being
+   * defined.
    */
   @Id(2)
   EntityRef get returnType;
 
+  /**
+   * The style of the typedef.
+   */
+  @Id(8)
+  TypedefStyle get style;
+
   /**
    * Type parameters of the typedef, if any.
    */
@@ -3058,6 +3295,17 @@ abstract class UnlinkedVariable extends base.SummaryClass {
   @Id(9)
   int get inferredTypeSlot;
 
+  /**
+   * If this is an instance non-final field, a nonzero slot id which is unique
+   * within this compilation unit.  If this id is found in
+   * [LinkedUnit.parametersInheritingCovariant], then the parameter of the
+   * synthetic setter inherits `@covariant` behavior from a base class.
+   *
+   * Otherwise, zero.
+   */
+  @Id(15)
+  int get inheritsCovariantSlot;
+
   /**
    * The synthetic initializer function of the variable.  Absent if the variable
    * does not have an initializer.
@@ -3127,6 +3375,7 @@ abstract class UnlinkedVariable extends base.SummaryClass {
   /**
    * If a local variable, the length of the visible range; zero otherwise.
    */
+  @deprecated
   @informative
   @Id(11)
   int get visibleLength;
@@ -3134,6 +3383,7 @@ abstract class UnlinkedVariable extends base.SummaryClass {
   /**
    * If a local variable, the beginning of the visible range; zero otherwise.
    */
+  @deprecated
   @informative
   @Id(12)
   int get visibleOffset;
