@@ -2,25 +2,26 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-library analyzer.src.task.driver;
-
 import 'dart:async';
 import 'dart:collection';
 
+import 'package:analyzer/exception/exception.dart';
 import 'package:analyzer/src/context/cache.dart';
 import 'package:analyzer/src/generated/engine.dart';
-import 'package:analyzer/src/generated/java_engine.dart';
 import 'package:analyzer/src/generated/resolver.dart';
 import 'package:analyzer/src/generated/utilities_general.dart';
+import 'package:analyzer/src/task/api/model.dart';
 import 'package:analyzer/src/task/inputs.dart';
 import 'package:analyzer/src/task/manager.dart';
-import 'package:analyzer/task/model.dart';
 
 final PerformanceTag analysisDriverProcessOutputs =
-    new PerformanceTag('AnalysisDriver.processOutputs');
+    _taskDriverTag.createChild('processOutputs');
 
 final PerformanceTag workOrderMoveNextPerformanceTag =
-    new PerformanceTag('WorkOrder.moveNext');
+    _taskDriverTag.createChild('workOrderMoveNext');
+
+final PerformanceTag _taskDriverTag =
+    PerformanceStatistics.analyzer.createChild('taskDriver');
 
 /**
  * An object that is used to cause analysis to be performed until all of the
@@ -226,7 +227,7 @@ class AnalysisDriver {
    */
   bool performAnalysisTask() {
     //
-    // TODO(brianwilkerson) This implementaiton does not allow us to prioritize
+    // TODO(brianwilkerson) This implementation does not allow us to prioritize
     // work across contexts. What we need is a way for an external client to ask
     // to have all priority files analyzed for each context, then ask for normal
     // files to be analyzed. There are a couple of ways to do this.
