@@ -13,14 +13,85 @@ part of dart.collection;
  * implementing `keys`, `operator[]`, `operator[]=`, `remove` and `clear`.
  * The remaining operations are implemented in terms of these five.
  *
- * The `keys` iterable should have efficient [length] and [contains]
- * operations, and it should catch concurrent modifications of the keys
- * while iterating.
+ * The `keys` iterable should have efficient [Iterable.length] and
+ * [Iterable.contains] operations, and it should catch concurrent modifications
+ * of the keys while iterating.
  *
  * A more efficient implementation is usually possible by overriding
  * some of the other members as well.
  */
-abstract class MapBase<K, V> = Object with MapMixin<K, V>;
+abstract class MapBase<K, V> extends MapMixin<K, V> {
+  static String mapToString(Map m) {
+    // Reuses the list in IterableBase for detecting toString cycles.
+    if (_isToStringVisiting(m)) {
+      return '{...}';
+    }
+
+    var result = new StringBuffer();
+    try {
+      _toStringVisiting.add(m);
+      result.write('{');
+      bool first = true;
+      m.forEach((k, v) {
+        if (!first) {
+          result.write(', ');
+        }
+        first = false;
+        result.write(k);
+        result.write(': ');
+        result.write(v);
+      });
+      result.write('}');
+    } finally {
+      assert(identical(_toStringVisiting.last, m));
+      _toStringVisiting.removeLast();
+    }
+
+    return result.toString();
+  }
+
+  static _id(x) => x;
+
+  /**
+   * Fills a [Map] with key/value pairs computed from [iterable].
+   *
+   * This method is used by [Map] classes in the named constructor
+   * `fromIterable`.
+   */
+  static void _fillMapWithMappedIterable(
+      Map map, Iterable iterable, key(element), value(element)) {
+    key ??= _id;
+    value ??= _id;
+
+    for (var element in iterable) {
+      map[key(element)] = value(element);
+    }
+  }
+
+  /**
+   * Fills a map by associating the [keys] to [values].
+   *
+   * This method is used by [Map] classes in the named constructor
+   * `fromIterables`.
+   */
+  static void _fillMapWithIterables(Map map, Iterable keys, Iterable values) {
+    Iterator keyIterator = keys.iterator;
+    Iterator valueIterator = values.iterator;
+
+    bool hasNextKey = keyIterator.moveNext();
+    bool hasNextValue = valueIterator.moveNext();
+
+    while (hasNextKey && hasNextValue) {
+      map[keyIterator.current] = valueIterator.current;
+      hasNextKey = keyIterator.moveNext();
+      hasNextValue = valueIterator.moveNext();
+    }
+
+    if (hasNextKey || hasNextValue) {
+      throw new ArgumentError("Iterables do not have same length.");
+    }
+  }
+}
 
 /**
  * Mixin implementing a [Map].
@@ -31,9 +102,9 @@ abstract class MapBase<K, V> = Object with MapMixin<K, V>;
  * implementing `keys`, `operator[]`, `operator[]=`, `remove` and `clear`.
  * The remaining operations are implemented in terms of these five.
  *
- * The `keys` iterable should have efficient [length] and [contains]
- * operations, and it should catch concurrent modifications of the keys
- * while iterating.
+ * The `keys` iterable should have efficient [Iterable.length] and
+ * [Iterable.contains] operations, and it should catch concurrent modifications
+ * of the keys while iterating.
  *
  * A more efficient implementation is usually possible by overriding
  * some of the other members as well.
@@ -47,6 +118,7 @@ abstract class MapMixin<K, V> implements Map<K, V> {
   // It should clear the map even if some keys are not equal to themselves.
   void clear();
 
+  Map<RK, RV> cast<RK, RV>() => Map.castFrom<K, V, RK, RV>(this);
   void forEach(void action(K key, V value)) {
     for (K key in keys) {
       action(key, this[key]);
@@ -73,12 +145,57 @@ abstract class MapMixin<K, V> implements Map<K, V> {
     return this[key] = ifAbsent();
   }
 
+  V update(K key, V update(V value), {V ifAbsent()}) {
+    if (this.containsKey(key)) {
+      return this[key] = update(this[key]);
+    }
+    if (ifAbsent != null) {
+      return this[key] = ifAbsent();
+    }
+    throw new ArgumentError.value(key, "key", "Key not in map.");
+  }
+
+  void updateAll(V update(K key, V value)) {
+    for (var key in this.keys) {
+      this[key] = update(key, this[key]);
+    }
+  }
+
+  Iterable<MapEntry<K, V>> get entries {
+    return keys.map((K key) => new MapEntry<K, V>(key, this[key]));
+  }
+
+  Map<K2, V2> map<K2, V2>(MapEntry<K2, V2> transform(K key, V value)) {
+    var result = <K2, V2>{};
+    for (var key in this.keys) {
+      var entry = transform(key, this[key]);
+      result[entry.key] = entry.value;
+    }
+    return result;
+  }
+
+  void addEntries(Iterable<MapEntry<K, V>> newEntries) {
+    for (var entry in newEntries) {
+      this[entry.key] = entry.value;
+    }
+  }
+
+  void removeWhere(bool test(K key, V value)) {
+    var keysToRemove = <K>[];
+    for (var key in keys) {
+      if (test(key, this[key])) keysToRemove.add(key);
+    }
+    for (var key in keysToRemove) {
+      this.remove(key);
+    }
+  }
+
   bool containsKey(Object key) => keys.contains(key);
   int get length => keys.length;
   bool get isEmpty => keys.isEmpty;
   bool get isNotEmpty => keys.isNotEmpty;
   Iterable<V> get values => new _MapBaseValueIterable<K, V>(this);
-  String toString() => Maps.mapToString(this);
+  String toString() => MapBase.mapToString(this);
 }
 
 /**
@@ -93,9 +210,9 @@ abstract class MapMixin<K, V> implements Map<K, V> {
  * The remaining non-modifying operations are implemented in terms of `keys`
  * and `operator[]`.
  *
- * The `keys` iterable should have efficient [length] and [contains]
- * operations, and it should catch concurrent modifications of the keys
- * while iterating.
+ * The `keys` iterable should have efficient [Iterable.length] and
+ * [Iterable.contains] operations, and it should catch concurrent modifications
+ * of the keys while iterating.
  *
  * A more efficient implementation is usually possible by overriding
  * some of the other members as well.
@@ -193,6 +310,7 @@ class MapView<K, V> implements Map<K, V> {
   final Map<K, V> _map;
   const MapView(Map<K, V> map) : _map = map;
 
+  Map<RK, RV> cast<RK, RV>() => _map.cast<RK, RV>();
   V operator [](Object key) => _map[key];
   void operator []=(K key, V value) {
     _map[key] = value;
@@ -220,6 +338,26 @@ class MapView<K, V> implements Map<K, V> {
   V remove(Object key) => _map.remove(key);
   String toString() => _map.toString();
   Iterable<V> get values => _map.values;
+
+  Iterable<MapEntry<K, V>> get entries => _map.entries;
+
+  void addEntries(Iterable<Object> entries) {
+    _map.addEntries(entries);
+  }
+
+  Map<K2, V2> map<K2, V2>(MapEntry<K2, V2> transform(K key, V value)) =>
+      _map.map<K2, V2>(transform);
+
+  V update(K key, V update(V value), {V ifAbsent()}) =>
+      _map.update(key, update, ifAbsent: ifAbsent);
+
+  void updateAll(V update(K key, V value)) {
+    _map.updateAll(update);
+  }
+
+  void removeWhere(bool test(K key, V value)) {
+    _map.removeWhere(test);
+  }
 }
 
 /**
@@ -229,147 +367,10 @@ class MapView<K, V> implements Map<K, V> {
  * the constructor, except for operations that modify the map.
  * Modifying operations throw instead.
  */
-class UnmodifiableMapView<K, V> = MapView<K, V>
-    with _UnmodifiableMapMixin<K, V>;
+class UnmodifiableMapView<K, V> extends MapView<K, V>
+    with _UnmodifiableMapMixin<K, V> {
+  UnmodifiableMapView(Map<K, V> map) : super(map);
 
-/**
- * Helper class which implements complex [Map] operations
- * in term of basic ones ([Map.keys], [Map.[]],
- * [Map.[]=] and [Map.remove].)  Not all methods are
- * necessary to implement each particular operation.
- */
-class Maps {
-  static bool containsValue(Map map, Object value) {
-    for (final v in map.values) {
-      if (v == value) {
-        return true;
-      }
-    }
-    return false;
-  }
-
-  static bool containsKey(Map map, Object key) {
-    for (final k in map.keys) {
-      if (k == key) {
-        return true;
-      }
-    }
-    return false;
-  }
-
-  static putIfAbsent(Map map, key, ifAbsent()) {
-    if (map.containsKey(key)) {
-      return map[key];
-    }
-    final v = ifAbsent();
-    map[key] = v;
-    return v;
-  }
-
-  static clear(Map map) {
-    for (final k in map.keys.toList()) {
-      map.remove(k);
-    }
-  }
-
-  static forEach(Map map, void f(key, value)) {
-    for (final k in map.keys) {
-      f(k, map[k]);
-    }
-  }
-
-  static Iterable getValues(Map map) {
-    return map.keys.map((key) => map[key]);
-  }
-
-  static int length(Map map) => map.keys.length;
-
-  static bool isEmpty(Map map) => map.keys.isEmpty;
-
-  static bool isNotEmpty(Map map) => map.keys.isNotEmpty;
-
-  /**
-   * Returns a string representing the specified map. The returned string
-   * looks like this: [:'{key0: value0, key1: value1, ... keyN: valueN}':].
-   * The value returned by its [toString] method is used to represent each
-   * key or value.
-   *
-   * If the map collection contains a reference to itself, either
-   * directly as a key or value, or indirectly through other collections
-   * or maps, the contained reference is rendered as [:'{...}':]. This
-   * prevents the infinite regress that would otherwise occur. So, for example,
-   * calling this method on a map whose sole entry maps the string key 'me'
-   * to a reference to the map would return [:'{me: {...}}':].
-   *
-   * A typical implementation of a map's [toString] method will
-   * simply return the results of this method applied to the collection.
-   */
-  static String mapToString(Map m) {
-    // Reuse the list in IterableBase for detecting toString cycles.
-    if (_isToStringVisiting(m)) {
-      return '{...}';
-    }
-
-    var result = new StringBuffer();
-    try {
-      _toStringVisiting.add(m);
-      result.write('{');
-      bool first = true;
-      m.forEach((k, v) {
-        if (!first) {
-          result.write(', ');
-        }
-        first = false;
-        result.write(k);
-        result.write(': ');
-        result.write(v);
-      });
-      result.write('}');
-    } finally {
-      assert(identical(_toStringVisiting.last, m));
-      _toStringVisiting.removeLast();
-    }
-
-    return result.toString();
-  }
-
-  static _id(x) => x;
-
-  /**
-   * Fills a map with key/value pairs computed from [iterable].
-   *
-   * This method is used by Map classes in the named constructor fromIterable.
-   */
-  static void _fillMapWithMappedIterable(
-      Map map, Iterable iterable, key(element), value(element)) {
-    if (key == null) key = _id;
-    if (value == null) value = _id;
-
-    for (var element in iterable) {
-      map[key(element)] = value(element);
-    }
-  }
-
-  /**
-   * Fills a map by associating the [keys] to [values].
-   *
-   * This method is used by Map classes in the named constructor fromIterables.
-   */
-  static void _fillMapWithIterables(Map map, Iterable keys, Iterable values) {
-    Iterator keyIterator = keys.iterator;
-    Iterator valueIterator = values.iterator;
-
-    bool hasNextKey = keyIterator.moveNext();
-    bool hasNextValue = valueIterator.moveNext();
-
-    while (hasNextKey && hasNextValue) {
-      map[keyIterator.current] = valueIterator.current;
-      hasNextKey = keyIterator.moveNext();
-      hasNextValue = valueIterator.moveNext();
-    }
-
-    if (hasNextKey || hasNextValue) {
-      throw new ArgumentError("Iterables do not have same length.");
-    }
-  }
+  Map<RK, RV> cast<RK, RV>() =>
+      new UnmodifiableMapView<RK, RV>(_map.cast<RK, RV>());
 }
