@@ -11,6 +11,8 @@
  * To use this library in your code:
  *
  *     import 'dart:isolate';
+ *
+ * {@category VM}
  */
 library dart.isolate;
 
@@ -67,9 +69,9 @@ class IsolateSpawnException implements Exception {
  */
 class Isolate {
   /** Argument to `ping` and `kill`: Ask for immediate action. */
-  static const int IMMEDIATE = 0;
+  static const int immediate = 0;
   /** Argument to `ping` and `kill`: Ask for action before the next event. */
-  static const int BEFORE_NEXT_EVENT = 1;
+  static const int beforeNextEvent = 1;
 
   /**
    * Control port used to send control messages to the isolate.
@@ -152,17 +154,16 @@ class Isolate {
   external static Isolate get current;
 
   /**
-   * Returns the package root of the current isolate, if any.
+   * The location of the package configuration of the current isolate, if any.
    *
-   * If the isolate is using a [packageConfig] or the isolate has not been
-   * setup for package resolution, this getter returns `null`, otherwise it
-   * returns the package root - a directory that package URIs are resolved
-   * against.
+   * This getter returns `null`, as the `packages/` directory is not supported
+   * in Dart 2.
    */
+  @Deprecated('packages/ directory resolution is not supported in Dart 2.')
   external static Future<Uri> get packageRoot;
 
   /**
-   * Returns the package root of the current isolate, if any.
+   * The package root of the current isolate, if any.
    *
    * If the isolate is using a [packageRoot] or the isolate has not been
    * setup for package resolution, this getter returns `null`, otherwise it
@@ -207,7 +208,7 @@ class Isolate {
    * as if by an initial call of `isolate.pause(isolate.pauseCapability)`.
    * To resume the isolate, call `isolate.resume(isolate.pauseCapability)`.
    *
-   * If the [errorAreFatal], [onExit] and/or [onError] parameters are provided,
+   * If the [errorsAreFatal], [onExit] and/or [onError] parameters are provided,
    * the isolate will act as if, respectively, [setErrorsFatal],
    * [addOnExitListener] and [addErrorListener] were called with the
    * corresponding parameter and was processed before the isolate starts
@@ -224,7 +225,8 @@ class Isolate {
    * Returns a future which will complete with an [Isolate] instance if the
    * spawning succeeded. It will complete with an error otherwise.
    */
-  external static Future<Isolate> spawn(void entryPoint(message), var message,
+  external static Future<Isolate> spawn<T>(
+      void entryPoint(T message), T message,
       {bool paused: false,
       bool errorsAreFatal,
       SendPort onExit,
@@ -252,7 +254,7 @@ class Isolate {
    * as if by an initial call of `isolate.pause(isolate.pauseCapability)`.
    * To resume the isolate, call `isolate.resume(isolate.pauseCapability)`.
    *
-   * If the [errorAreFatal], [onExit] and/or [onError] parameters are provided,
+   * If the [errorsAreFatal], [onExit] and/or [onError] parameters are provided,
    * the isolate will act as if, respectively, [setErrorsFatal],
    * [addOnExitListener] and [addErrorListener] were called with the
    * corresponding parameter and was processed before the isolate starts
@@ -264,10 +266,13 @@ class Isolate {
    * before those methods can complete.
    *
    * If the [checked] parameter is set to `true` or `false`,
-   * the new isolate will run code in checked mode,
-   * respectively in production mode, if possible.
-   * If the parameter is omitted, the new isolate will inherit the
-   * value from the current isolate.
+   * the new isolate will run code in checked mode (enabling asserts and type
+   * checks), respectively in production mode (disabling asserts and type
+   * checks), if possible. If the parameter is omitted, the new isolate will
+   * inherit the value from the current isolate.
+   *
+   * In Dart2 strong mode, the `checked` parameter only controls asserts, but
+   * not type checks.
    *
    * It may not always be possible to honor the `checked` parameter.
    * If the isolate code was pre-compiled, it may not be possible to change
@@ -276,16 +281,6 @@ class Isolate {
    *
    * WARNING: The [checked] parameter is not implemented on all platforms yet.
    *
-   * If the [packageRoot] parameter is provided, it is used to find the location
-   * of package sources in the spawned isolate.
-   *
-   * The `packageRoot` URI must be a "file" or "http"/"https" URI that specifies
-   * a directory. If it doesn't end in a slash, one will be added before
-   * using the URI, and any query or fragment parts are ignored.
-   * Package imports (like `"package:foo/bar.dart"`) in the new isolate are
-   * resolved against this location, as by
-   * `packageRoot.resolve("foo/bar.dart")`.
-   *
    * If the [packageConfig] parameter is provided, then it is used to find the
    * location of a package resolution configuration file for the spawned
    * isolate.
@@ -307,14 +302,17 @@ class Isolate {
    * spawning succeeded. It will complete with an error otherwise.
    */
   external static Future<Isolate> spawnUri(
-      Uri uri, List<String> args, var message,
+      Uri uri,
+      List<String> args,
+      var message,
       {bool paused: false,
       SendPort onExit,
       SendPort onError,
       bool errorsAreFatal,
       bool checked,
       Map<String, String> environment,
-      Uri packageRoot,
+      @Deprecated('The packages/ dir is not supported in Dart 2')
+          Uri packageRoot,
       Uri packageConfig,
       bool automaticPackageResolution: false});
 
@@ -376,7 +374,7 @@ class Isolate {
   external void resume(Capability resumeCapability);
 
   /**
-   * Requests an exist message on [responsePort] when the isolate terminates.
+   * Requests an exit message on [responsePort] when the isolate terminates.
    *
    * The isolate will send [response] as a message on [responsePort] as the last
    * thing before it terminates. It will run no further code after the message
@@ -407,7 +405,7 @@ class Isolate {
   external void addOnExitListener(SendPort responsePort, {Object response});
 
   /**
-   * Stop listening for exit messages from the isolate.
+   * Stops listening for exit messages from the isolate.
    *
    * Requests for the isolate to not send exit messages on [responsePort].
    * If the isolate isn't expecting to send exit messages on [responsePort],
@@ -416,10 +414,11 @@ class Isolate {
    *
    * If the same port has been passed via [addOnExitListener] more than once,
    * only one call to `removeOnExitListener` is needed to stop it from receiving
-   * exit messagees.
+   * exit messages.
    *
-   * Closing the receive port at the end of the send port will not stop the
-   * isolate from sending exit messages, they are just going to be lost.
+   * Closing the receive port that is associated with the [responsePort] does
+   * not stop the isolate from sending uncaught errors, they are just going to
+   * be lost.
    *
    * An exit message may still be sent if the isolate terminates
    * before this request is received and processed.
@@ -427,7 +426,7 @@ class Isolate {
   external void removeOnExitListener(SendPort responsePort);
 
   /**
-   * Set whether uncaught errors will terminate the isolate.
+   * Sets whether uncaught errors will terminate the isolate.
    *
    * If errors are fatal, any uncaught error will terminate the isolate
    * event loop and shut down the isolate.
@@ -435,8 +434,9 @@ class Isolate {
    * This call requires the [terminateCapability] for the isolate.
    * If the capability is absent or incorrect, no change is made.
    *
-   * Since isolates run concurrently, it's possible for it to exit due to an
-   * error before errors are set non-fatal.
+   * Since isolates run concurrently, it's possible for the receiving isolate
+   * to exit due to an error, before a request, using this method, has been
+   * received and processed.
    * To avoid this, either use the corresponding parameter to the spawn
    * function, or start the isolate paused, set errors non-fatal and
    * then resume the isolate.
@@ -449,18 +449,18 @@ class Isolate {
    * The isolate is requested to terminate itself.
    * The [priority] argument specifies when this must happen.
    *
-   * The [priority], when provided, must be one of [IMMEDIATE] or
-   * [BEFORE_NEXT_EVENT] (the default).
+   * The [priority], when provided, must be one of [immediate] or
+   * [beforeNextEvent] (the default).
    * The shutdown is performed at different times depending on the priority:
    *
-   * * `IMMEDIATE`: The isolate shuts down as soon as possible.
+   * * `immediate`: The isolate shuts down as soon as possible.
    *     Control messages are handled in order, so all previously sent control
    *     events from this isolate will all have been processed.
    *     The shutdown should happen no later than if sent with
-   *     `BEFORE_NEXT_EVENT`.
+   *     `beforeNextEvent`.
    *     It may happen earlier if the system has a way to shut down cleanly
    *     at an earlier time, even during the execution of another event.
-   * * `BEFORE_NEXT_EVENT`: The shutdown is scheduled for the next time
+   * * `beforeNextEvent`: The shutdown is scheduled for the next time
    *     control returns to the event loop of the receiving isolate,
    *     after the current event, and any already scheduled control events,
    *     are completed.
@@ -469,10 +469,10 @@ class Isolate {
    * of the isolate identified by [controlPort],
    * the kill request is ignored by the receiving isolate.
    */
-  external void kill({int priority: BEFORE_NEXT_EVENT});
+  external void kill({int priority: beforeNextEvent});
 
   /**
-   * Request that the isolate send [response] on the [responsePort].
+   * Requests that the isolate send [response] on the [responsePort].
    *
    * The [response] object must follow the same restrictions as enforced by
    * [SendPort.send].
@@ -482,20 +482,20 @@ class Isolate {
    * If the isolate is alive, it will eventually send `response`
    * (defaulting to `null`) on the response port.
    *
-   * The [priority] must be one of [IMMEDIATE] or [BEFORE_NEXT_EVENT].
+   * The [priority] must be one of [immediate] or [beforeNextEvent].
    * The response is sent at different times depending on the ping type:
    *
-   * * `IMMEDIATE`: The isolate responds as soon as it receives the
+   * * `immediate`: The isolate responds as soon as it receives the
    *     control message. This is after any previous control message
    *     from the same isolate has been received and processed,
    *     but may be during execution of another event.
-   * * `BEFORE_NEXT_EVENT`: The response is scheduled for the next time
+   * * `beforeNextEvent`: The response is scheduled for the next time
    *     control returns to the event loop of the receiving isolate,
    *     after the current event, and any already scheduled control events,
    *     are completed.
    */
   external void ping(SendPort responsePort,
-      {Object response, int priority: IMMEDIATE});
+      {Object response, int priority: immediate});
 
   /**
    * Requests that uncaught errors of the isolate are sent back to [port].
@@ -510,6 +510,10 @@ class Isolate {
    * Listening using the same port more than once does nothing.
    * A port will only receive each error once,
    * and will only need to be removed once using [removeErrorListener].
+
+   * Closing the receive port that is associated with the port does not stop
+   * the isolate from sending uncaught errors, they are just going to be lost.
+   * Instead use [removeErrorListener] to stop receiving errors on [port].
    *
    * Since isolates run concurrently, it's possible for it to exit before the
    * error listener is established. To avoid this, start the isolate paused,
@@ -518,19 +522,16 @@ class Isolate {
   external void addErrorListener(SendPort port);
 
   /**
-   * Stop listening for uncaught errors from the isolate.
+   * Stops listening for uncaught errors from the isolate.
    *
-   * Requests for the isolate to not send uncaught errors on [responsePort].
-   * If the isolate isn't expecting to send uncaught errors on [responsePort],
+   * Requests for the isolate to not send uncaught errors on [port].
+   * If the isolate isn't expecting to send uncaught errors on [port],
    * because the port hasn't been added using [addErrorListener],
    * or because it has already been removed, the request is ignored.
    *
    * If the same port has been passed via [addErrorListener] more than once,
    * only one call to `removeErrorListener` is needed to stop it from receiving
-   * unaught errors.
-   *
-   * Closing the receive port at the end of the send port will not stop the
-   * isolate from sending uncaught errors, they are just going to be lost.
+   * uncaught errors.
    *
    * Uncaught errors message may still be sent by the isolate
    * until this request is received and processed.
@@ -689,12 +690,12 @@ abstract class RawReceivePort {
    * can not be paused. The data-handler must be set before the first
    * event is received.
    */
-  external factory RawReceivePort([void handler(event)]);
+  external factory RawReceivePort([Function handler]);
 
   /**
    * Sets the handler that is invoked for every incoming message.
    *
-   * The handler is invoked in the root-zone ([Zone.ROOT]).
+   * The handler is invoked in the root-zone ([Zone.root]).
    */
   void set handler(Function newHandler);
 
