@@ -12,38 +12,79 @@ import '../visitor.dart';
 
 import 'async.dart';
 
-Program transformProgram(Program program) {
-  var helper = new HelperNodes.fromProgram(program);
-  var rewriter = new RecursiveContinuationRewriter(helper);
-  return rewriter.rewriteProgram(program);
+class ContinuationVariables {
+  static const awaitJumpVar = ':await_jump_var';
+  static const awaitContextVar = ':await_ctx_var';
+  static const exceptionParam = ':exception';
+  static const stackTraceParam = ':stack_trace';
+
+  static String savedTryContextVar(int depth) => ':saved_try_context_var$depth';
+  static String exceptionVar(int depth) => ':exception$depth';
+  static String stackTraceVar(int depth) => ':stack_trace$depth';
+}
+
+void transformLibraries(
+    CoreTypes coreTypes, List<Library> libraries, bool syncAsync) {
+  var helper = new HelperNodes.fromCoreTypes(coreTypes);
+  var rewriter = new RecursiveContinuationRewriter(helper, syncAsync);
+  for (var library in libraries) {
+    rewriter.rewriteLibrary(library);
+  }
+}
+
+Component transformComponent(
+    CoreTypes coreTypes, Component component, bool syncAsync) {
+  var helper = new HelperNodes.fromCoreTypes(coreTypes);
+  var rewriter = new RecursiveContinuationRewriter(helper, syncAsync);
+  return rewriter.rewriteComponent(component);
+}
+
+Procedure transformProcedure(
+    CoreTypes coreTypes, Procedure procedure, bool syncAsync) {
+  var helper = new HelperNodes.fromCoreTypes(coreTypes);
+  var rewriter = new RecursiveContinuationRewriter(helper, syncAsync);
+  return rewriter.visitProcedure(procedure);
 }
 
 class RecursiveContinuationRewriter extends Transformer {
   final HelperNodes helper;
+
+  /// Whether `async` functions should start synchronously.
+  final bool syncAsync;
+
   final VariableDeclaration asyncJumpVariable = new VariableDeclaration(
-      ":await_jump_var",
+      ContinuationVariables.awaitJumpVar,
       initializer: new IntLiteral(0));
   final VariableDeclaration asyncContextVariable =
-      new VariableDeclaration(":await_ctx_var");
+      new VariableDeclaration(ContinuationVariables.awaitContextVar);
 
-  RecursiveContinuationRewriter(this.helper);
+  RecursiveContinuationRewriter(this.helper, this.syncAsync);
 
-  Program rewriteProgram(Program node) {
+  Component rewriteComponent(Component node) {
     return node.accept(this);
   }
 
+  Library rewriteLibrary(Library node) {
+    return node.accept(this);
+  }
+
+  visitProcedure(Procedure node) {
+    return node.isAbstract ? node : super.visitProcedure(node);
+  }
+
   visitFunctionNode(FunctionNode node) {
     switch (node.asyncMarker) {
       case AsyncMarker.Sync:
       case AsyncMarker.SyncYielding:
-        node.transformChildren(new RecursiveContinuationRewriter(helper));
+        node.transformChildren(
+            new RecursiveContinuationRewriter(helper, syncAsync));
         return node;
       case AsyncMarker.SyncStar:
-        return new SyncStarFunctionRewriter(helper, node).rewrite();
+        return new SyncStarFunctionRewriter(helper, node, syncAsync).rewrite();
       case AsyncMarker.Async:
-        return new AsyncFunctionRewriter(helper, node).rewrite();
+        return new AsyncFunctionRewriter(helper, node, syncAsync).rewrite();
       case AsyncMarker.AsyncStar:
-        return new AsyncStarFunctionRewriter(helper, node).rewrite();
+        return new AsyncStarFunctionRewriter(helper, node, syncAsync).rewrite();
     }
   }
 }
@@ -56,8 +97,35 @@ abstract class ContinuationRewriterBase extends RecursiveContinuationRewriter {
   int capturedTryDepth = 0; // Deepest yield point within a try-block.
   int capturedCatchDepth = 0; // Deepest yield point within a catch-block.
 
-  ContinuationRewriterBase(HelperNodes helper, this.enclosingFunction)
-      : super(helper);
+  ContinuationRewriterBase(
+      HelperNodes helper, this.enclosingFunction, bool syncAsync)
+      : super(helper, syncAsync);
+
+  /// Given a container [type], which is an instantiation of the given
+  /// [containerClass] extract its element type.
+  ///
+  /// This is used to extract element type from Future<T>, Iterable<T> and
+  /// Stream<T> instantiations.
+  ///
+  /// If instantiation is not valid (has more than 1 type argument) then
+  /// this function returns [InvalidType].
+  static DartType elementTypeFrom(Class containerClass, DartType type) {
+    if (type is InterfaceType) {
+      if (type.classNode == containerClass) {
+        if (type.typeArguments.length == 0) {
+          return const DynamicType();
+        } else if (type.typeArguments.length == 1) {
+          return type.typeArguments[0];
+        } else {
+          return const InvalidType();
+        }
+      }
+    }
+    return const DynamicType();
+  }
+
+  DartType elementTypeFromReturnType(Class expected) =>
+      elementTypeFrom(expected, enclosingFunction.returnType);
 
   Statement createContinuationPoint([Expression value]) {
     if (value == null) value = new NullLiteral();
@@ -97,13 +165,15 @@ abstract class ContinuationRewriterBase extends RecursiveContinuationRewriter {
   }
 
   Iterable<VariableDeclaration> createCapturedTryVariables() =>
-      new Iterable.generate(capturedTryDepth,
-          (depth) => new VariableDeclaration(":saved_try_context_var${depth}"));
+      new Iterable.generate(
+          capturedTryDepth,
+          (depth) => new VariableDeclaration(
+              ContinuationVariables.savedTryContextVar(depth)));
 
   Iterable<VariableDeclaration> createCapturedCatchVariables() =>
       new Iterable.generate(capturedCatchDepth).expand((depth) => [
-            new VariableDeclaration(":exception${depth}"),
-            new VariableDeclaration(":stack_trace${depth}"),
+            new VariableDeclaration(ContinuationVariables.exceptionVar(depth)),
+            new VariableDeclaration(ContinuationVariables.stackTraceVar(depth)),
           ]);
 
   List<VariableDeclaration> variableDeclarations() =>
@@ -115,13 +185,17 @@ abstract class ContinuationRewriterBase extends RecursiveContinuationRewriter {
 class SyncStarFunctionRewriter extends ContinuationRewriterBase {
   final VariableDeclaration iteratorVariable;
 
-  SyncStarFunctionRewriter(helper, enclosingFunction)
+  SyncStarFunctionRewriter(
+      HelperNodes helper, FunctionNode enclosingFunction, syncAsync)
       : iteratorVariable = new VariableDeclaration(':iterator')
-          ..type = helper.iteratorClass.rawType,
-        super(helper, enclosingFunction);
+          ..type = new InterfaceType(helper.syncIteratorClass, [
+            ContinuationRewriterBase.elementTypeFrom(
+                helper.iterableClass, enclosingFunction.returnType)
+          ]),
+        super(helper, enclosingFunction, syncAsync);
 
   FunctionNode rewrite() {
-    // :sync_body(:iterator) {
+    // :sync_op(:iterator) {
     //     modified <node.body>;
     // }
 
@@ -142,8 +216,9 @@ class SyncStarFunctionRewriter extends ContinuationRewriterBase {
         new FunctionDeclaration(nestedClosureVariable, function)
           ..fileOffset = enclosingFunction.parent.fileOffset;
 
-    // return new _SyncIterable(:sync_body);
-    final arguments = new Arguments([new VariableGet(nestedClosureVariable)]);
+    // return new _SyncIterable<T>(:sync_body);
+    final arguments = new Arguments([new VariableGet(nestedClosureVariable)],
+        types: [elementTypeFromReturnType(helper.iterableClass)]);
     final returnStatement = new ReturnStatement(
         new ConstructorInvocation(helper.syncIterableConstructor, arguments));
 
@@ -171,19 +246,19 @@ class SyncStarFunctionRewriter extends ContinuationRewriterBase {
 
     var statements = <Statement>[];
     if (node.isYieldStar) {
-      var markYieldEach = new ExpressionStatement(new PropertySet(
+      statements.add(new ExpressionStatement(new PropertySet(
           new VariableGet(iteratorVariable),
-          new Name("isYieldEach", helper.coreLibrary),
-          new BoolLiteral(true)));
-      statements.add(markYieldEach);
+          new Name("_yieldEachIterable", helper.coreLibrary),
+          transformedExpression,
+          helper.syncIteratorYieldEachIterable)));
+    } else {
+      statements.add(new ExpressionStatement(new PropertySet(
+          new VariableGet(iteratorVariable),
+          new Name("_current", helper.coreLibrary),
+          transformedExpression,
+          helper.syncIteratorCurrent)));
     }
 
-    var setCurrentIteratorValue = new ExpressionStatement(new PropertySet(
-        new VariableGet(iteratorVariable),
-        new Name("_current", helper.coreLibrary),
-        transformedExpression));
-
-    statements.add(setCurrentIteratorValue);
     statements.add(createContinuationPoint(new BoolLiteral(true)));
     return new Block(statements);
   }
@@ -199,6 +274,8 @@ class SyncStarFunctionRewriter extends ContinuationRewriterBase {
 abstract class AsyncRewriterBase extends ContinuationRewriterBase {
   final VariableDeclaration nestedClosureVariable =
       new VariableDeclaration(":async_op");
+  final VariableDeclaration stackTraceVariable =
+      new VariableDeclaration(":async_stack_trace");
   final VariableDeclaration thenContinuationVariable =
       new VariableDeclaration(":async_op_then");
   final VariableDeclaration catchErrorContinuationVariable =
@@ -208,12 +285,16 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
 
   ExpressionLifter expressionRewriter;
 
-  AsyncRewriterBase(helper, enclosingFunction)
-      : super(helper, enclosingFunction) {}
+  AsyncRewriterBase(
+      HelperNodes helper, FunctionNode enclosingFunction, bool syncAsync)
+      : super(helper, enclosingFunction, syncAsync) {}
 
   void setupAsyncContinuations(List<Statement> statements) {
     expressionRewriter = new ExpressionLifter(this);
 
+    // var :async_stack_trace;
+    statements.add(stackTraceVariable);
+
     // var :async_op_then;
     statements.add(thenContinuationVariable);
 
@@ -225,8 +306,8 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
     // }
     final parameters = <VariableDeclaration>[
       expressionRewriter.asyncResult,
-      new VariableDeclaration(':exception'),
-      new VariableDeclaration(':stack_trace'),
+      new VariableDeclaration(ContinuationVariables.exceptionParam),
+      new VariableDeclaration(ContinuationVariables.stackTraceParam),
     ];
 
     // Note: SyncYielding functions have no Dart equivalent. Since they are
@@ -253,6 +334,13 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
           ..fileOffset = enclosingFunction.parent.fileOffset;
     statements.add(closureFunction);
 
+    // :async_stack_trace = _asyncStackTraceHelper(asyncBody);
+    final stackTrace = new StaticInvocation(helper.asyncStackTraceHelper,
+        new Arguments(<Expression>[new VariableGet(nestedClosureVariable)]));
+    final stackTraceAssign = new ExpressionStatement(
+        new VariableSet(stackTraceVariable, stackTrace));
+    statements.add(stackTraceAssign);
+
     // :async_op_then = _asyncThenWrapperHelper(asyncBody);
     final boundThenClosure = new StaticInvocation(helper.asyncThenWrapper,
         new Arguments(<Expression>[new VariableGet(nestedClosureVariable)]));
@@ -280,14 +368,18 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
     var exceptionVariable = new VariableDeclaration(":exception");
     var stackTraceVariable = new VariableDeclaration(":stack_trace");
 
-    return new TryCatch(buildReturn(labeledBody), <Catch>[
-      new Catch(
-          exceptionVariable,
-          new Block(<Statement>[
-            buildCatchBody(exceptionVariable, stackTraceVariable)
-          ]),
-          stackTrace: stackTraceVariable)
-    ]);
+    return new TryCatch(
+      buildReturn(labeledBody),
+      <Catch>[
+        new Catch(
+            exceptionVariable,
+            new Block(<Statement>[
+              buildCatchBody(exceptionVariable, stackTraceVariable)
+            ]),
+            stackTrace: stackTraceVariable)
+      ],
+      isSynthetic: true,
+    );
   }
 
   Statement buildCatchBody(
@@ -297,11 +389,6 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
 
   List<Statement> statements = <Statement>[];
 
-  TreeNode visitInvalidStatement(InvalidStatement stmt) {
-    statements.add(stmt);
-    return null;
-  }
-
   TreeNode visitExpressionStatement(ExpressionStatement stmt) {
     stmt.expression = expressionRewriter.rewrite(stmt.expression, statements)
       ..parent = stmt;
@@ -325,8 +412,73 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
     return null;
   }
 
+  TreeNode visitAssertBlock(AssertBlock stmt) {
+    var saved = statements;
+    statements = <Statement>[];
+    for (var statement in stmt.statements) {
+      statement.accept(this);
+    }
+    saved.add(new Block(statements));
+    statements = saved;
+    return null;
+  }
+
   TreeNode visitAssertStatement(AssertStatement stmt) {
-    // TODO!
+    var condEffects = <Statement>[];
+    var cond = expressionRewriter.rewrite(stmt.condition, condEffects);
+    if (stmt.message == null) {
+      stmt.condition = cond..parent = stmt;
+      // If the translation of the condition produced a non-empty list of
+      // statements, ensure they are guarded by whether asserts are enabled.
+      statements.add(
+          condEffects.isEmpty ? stmt : new AssertBlock(condEffects..add(stmt)));
+      return null;
+    }
+
+    // The translation depends on the translation of the message, by cases.
+    Statement result;
+    var msgEffects = <Statement>[];
+    stmt.message = expressionRewriter.rewrite(stmt.message, msgEffects)
+      ..parent = stmt;
+    if (condEffects.isEmpty) {
+      if (msgEffects.isEmpty) {
+        // The condition rewrote to ([], C) and the message rewrote to ([], M).
+        // The result is
+        //
+        // assert(C, M)
+        stmt.condition = cond..parent = stmt;
+        result = stmt;
+      } else {
+        // The condition rewrote to ([], C) and the message rewrote to (S*, M)
+        // where S* is non-empty.  The result is
+        //
+        // assert { if (C) {} else { S*; assert(false, M); }}
+        stmt.condition = new BoolLiteral(false)..parent = stmt;
+        result = new AssertBlock([
+          new IfStatement(
+              cond, new EmptyStatement(), new Block(msgEffects..add(stmt)))
+        ]);
+      }
+    } else {
+      if (msgEffects.isEmpty) {
+        // The condition rewrote to (S*, C) where S* is non-empty and the
+        // message rewrote to ([], M).  The result is
+        //
+        // assert { S*; assert(C, M); }
+        stmt.condition = cond..parent = stmt;
+        condEffects.add(stmt);
+      } else {
+        // The condition rewrote to (S0*, C) and the message rewrote to (S1*, M)
+        // where both S0* and S1* are non-empty.  The result is
+        //
+        // assert { S0*; if (C) {} else { S1*; assert(false, M); }}
+        stmt.condition = new BoolLiteral(false)..parent = stmt;
+        condEffects.add(new IfStatement(
+            cond, new EmptyStatement(), new Block(msgEffects..add(stmt))));
+      }
+      result = new AssertBlock(condEffects);
+    }
+    statements.add(result);
     return null;
   }
 
@@ -410,7 +562,8 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
       initEffects[i] = <Statement>[];
       if (decl.initializer != null) {
         decl.initializer = expressionRewriter.rewrite(
-            decl.initializer, initEffects[i])..parent = decl;
+            decl.initializer, initEffects[i])
+          ..parent = decl;
       }
       isSimple = isSimple && initEffects[i].isEmpty;
     }
@@ -420,7 +573,8 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
     for (int i = 0; i < length; ++i) {
       updateEffects[i] = <Statement>[];
       stmt.updates[i] = expressionRewriter.rewrite(
-          stmt.updates[i], updateEffects[i])..parent = stmt;
+          stmt.updates[i], updateEffects[i])
+        ..parent = stmt;
       isSimple = isSimple && updateEffects[i].isEmpty;
     }
 
@@ -530,8 +684,9 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
     }
     loopBody.add(
         new IfStatement(cond, new Block(newBody), new BreakStatement(labeled)));
-    labeled.body = new WhileStatement(
-        new BoolLiteral(true), new Block(loopBody))..parent = labeled;
+    labeled.body =
+        new WhileStatement(new BoolLiteral(true), new Block(loopBody))
+          ..parent = labeled;
     statements.add(new Block(<Statement>[]
       ..addAll(temps)
       ..add(labeled)));
@@ -542,46 +697,66 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
     if (stmt.isAsync) {
       // Transform
       //
-      //   await for (var variable in <stream-expression>) { ... }
+      //   await for (T variable in <stream-expression>) { ... }
       //
       // To:
       //
       //   {
-      //     var :for-iterator = new StreamIterator(<stream-expression>);
+      //     _StreamIterator<T> :for-iterator =
+      //         new _StreamIterator<T>(<stream-expression>);
       //     try {
       //       while (await :for-iterator.moveNext()) {
-      //         var <variable> = :for-iterator.current;
+      //         T <variable> = :for-iterator.current;
       //         ...
       //       }
       //     } finally {
-      //       await :for-iterator.cancel();
+      //       if (:for-iterator._subscription != null) await :for-iterator.cancel();
       //     }
       //   }
+      var valueVariable = stmt.variable;
+
       var iteratorVariable = new VariableDeclaration(':for-iterator',
           initializer: new ConstructorInvocation(
               helper.streamIteratorConstructor,
               new Arguments(<Expression>[stmt.iterable],
-                  types: [const DynamicType()])));
+                  types: [valueVariable.type])),
+          type: new InterfaceType(
+              helper.streamIteratorClass, [valueVariable.type]));
 
-      // await iterator.moveNext()
+      // await :for-iterator.moveNext()
       var condition = new AwaitExpression(new MethodInvocation(
           new VariableGet(iteratorVariable),
           new Name('moveNext'),
-          new Arguments(<Expression>[])))..fileOffset = stmt.fileOffset;
+          new Arguments(<Expression>[]),
+          helper.streamIteratorMoveNext))
+        ..fileOffset = stmt.fileOffset;
 
-      // var <variable> = iterator.current;
-      var valueVariable = stmt.variable;
+      // T <variable> = :for-iterator.current;
       valueVariable.initializer = new PropertyGet(
-          new VariableGet(iteratorVariable), new Name('current'));
+          new VariableGet(iteratorVariable),
+          new Name('current'),
+          helper.streamIteratorCurrent);
       valueVariable.initializer.parent = valueVariable;
 
       var whileBody = new Block(<Statement>[valueVariable, stmt.body]);
       var tryBody = new WhileStatement(condition, whileBody);
 
-      // iterator.cancel();
-      var tryFinalizer = new ExpressionStatement(new AwaitExpression(
-          new MethodInvocation(new VariableGet(iteratorVariable),
-              new Name('cancel'), new Arguments(<Expression>[]))));
+      // if (:for-iterator._subscription != null) await :for-iterator.cancel();
+      var tryFinalizer = new IfStatement(
+          new Not(new MethodInvocation(
+              new PropertyGet(
+                  new VariableGet(iteratorVariable),
+                  new Name("_subscription", helper.asyncLibrary),
+                  helper.coreTypes.streamIteratorSubscription),
+              new Name("=="),
+              new Arguments([new NullLiteral()]),
+              helper.coreTypes.objectEquals)),
+          new ExpressionStatement(new AwaitExpression(new MethodInvocation(
+              new VariableGet(iteratorVariable),
+              new Name('cancel'),
+              new Arguments(<Expression>[]),
+              helper.streamIteratorCancel))),
+          null);
 
       var tryFinally = new TryFinally(tryBody, tryFinalizer);
 
@@ -659,7 +834,8 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
   TreeNode visitVariableDeclaration(VariableDeclaration stmt) {
     if (stmt.initializer != null) {
       stmt.initializer = expressionRewriter.rewrite(
-          stmt.initializer, statements)..parent = stmt;
+          stmt.initializer, statements)
+        ..parent = stmt;
     }
     statements.add(stmt);
     return null;
@@ -677,32 +853,40 @@ abstract class AsyncRewriterBase extends ContinuationRewriterBase {
 class AsyncStarFunctionRewriter extends AsyncRewriterBase {
   VariableDeclaration controllerVariable;
 
-  AsyncStarFunctionRewriter(helper, enclosingFunction)
-      : super(helper, enclosingFunction);
+  AsyncStarFunctionRewriter(
+      HelperNodes helper, FunctionNode enclosingFunction, bool syncAsync)
+      : super(helper, enclosingFunction, syncAsync);
 
   FunctionNode rewrite() {
     var statements = <Statement>[];
 
-    // var :controller;
-    controllerVariable = new VariableDeclaration(":controller");
+    final elementType = elementTypeFromReturnType(helper.streamClass);
+
+    // _AsyncStarStreamController<T> :controller;
+    controllerVariable = new VariableDeclaration(":controller",
+        type: new InterfaceType(
+            helper.asyncStarStreamControllerClass, [elementType]));
     statements.add(controllerVariable);
 
     setupAsyncContinuations(statements);
 
-    // :controller = new _AsyncController(:async_op);
-    var arguments =
-        new Arguments(<Expression>[new VariableGet(nestedClosureVariable)]);
-    var buildController =
-        new ConstructorInvocation(helper.streamControllerConstructor, arguments)
-          ..fileOffset = enclosingFunction.fileOffset;
+    // :controller = new _AsyncStarStreamController<T>(:async_op);
+    var arguments = new Arguments(
+        <Expression>[new VariableGet(nestedClosureVariable)],
+        types: [elementType]);
+    var buildController = new ConstructorInvocation(
+        helper.asyncStarStreamControllerConstructor, arguments)
+      ..fileOffset = enclosingFunction.fileOffset;
     var setController = new ExpressionStatement(
         new VariableSet(controllerVariable, buildController));
     statements.add(setController);
 
     // return :controller.stream;
     var completerGet = new VariableGet(controllerVariable);
-    var returnStatement = new ReturnStatement(
-        new PropertyGet(completerGet, new Name('stream', helper.asyncLibrary)));
+    var returnStatement = new ReturnStatement(new PropertyGet(
+        completerGet,
+        new Name('stream', helper.asyncLibrary),
+        helper.asyncStarStreamControllerStream));
     statements.add(returnStatement);
 
     enclosingFunction.body = new Block(statements);
@@ -718,8 +902,9 @@ class AsyncStarFunctionRewriter extends AsyncRewriterBase {
 
     var finallyBody = new ExpressionStatement(new MethodInvocation(
         new VariableGet(controllerVariable),
-        new Name("close", helper.asyncLibrary),
-        new Arguments(<Expression>[])));
+        new Name("close"),
+        new Arguments(<Expression>[]),
+        helper.asyncStarStreamControllerClose));
 
     var tryFinally = new TryFinally(body, new Block(<Statement>[finallyBody]));
     return tryFinally;
@@ -728,11 +913,12 @@ class AsyncStarFunctionRewriter extends AsyncRewriterBase {
   Statement buildCatchBody(exceptionVariable, stackTraceVariable) {
     return new ExpressionStatement(new MethodInvocation(
         new VariableGet(controllerVariable),
-        new Name("addError", helper.asyncLibrary),
+        new Name("addError"),
         new Arguments(<Expression>[
           new VariableGet(exceptionVariable),
           new VariableGet(stackTraceVariable)
-        ])));
+        ]),
+        helper.asyncStarStreamControllerAddError));
   }
 
   Statement buildReturn(Statement body) {
@@ -750,7 +936,11 @@ class AsyncStarFunctionRewriter extends AsyncRewriterBase {
     var addExpression = new MethodInvocation(
         new VariableGet(controllerVariable),
         new Name(stmt.isYieldStar ? 'addStream' : 'add', helper.asyncLibrary),
-        new Arguments(<Expression>[expr]))..fileOffset = stmt.fileOffset;
+        new Arguments(<Expression>[expr]),
+        stmt.isYieldStar
+            ? helper.asyncStarStreamControllerAddStream
+            : helper.asyncStarStreamControllerAdd)
+      ..fileOffset = stmt.fileOffset;
 
     statements.add(new IfStatement(
         addExpression,
@@ -772,39 +962,48 @@ class AsyncFunctionRewriter extends AsyncRewriterBase {
   VariableDeclaration completerVariable;
   VariableDeclaration returnVariable;
 
-  AsyncFunctionRewriter(helper, enclosingFunction)
-      : super(helper, enclosingFunction);
+  AsyncFunctionRewriter(
+      HelperNodes helper, FunctionNode enclosingFunction, bool syncAsync)
+      : super(helper, enclosingFunction, syncAsync);
 
   FunctionNode rewrite() {
     var statements = <Statement>[];
 
-    // The original function return type should be Future<T> because the
-    // function is async. If it was, we make a Completer<T>.  Otherwise
-    // We will make a malformed type.
-    var future_type = enclosingFunction.returnType;
-    DartType returnType = const DynamicType();
-    if (future_type is InterfaceType) {
-      if (future_type.classNode == helper.futureClass) {
-        if (future_type.typeArguments.length == 0) {
-          returnType = const DynamicType();
-        } else if (future_type.typeArguments.length == 1) {
-          returnType = future_type.typeArguments[0];
-        } else {
-          returnType = const InvalidType();
-        }
-      }
+    // The original function return type should be Future<T> or FutureOr<T>
+    // because the function is async. If it was, we make a Completer<T>,
+    // otherwise we make a malformed type.  In a "Future<T> foo() async {}"
+    // function the body can either return a "T" or a "Future<T>" => a
+    // "FutureOr<T>".
+    DartType valueType = elementTypeFromReturnType(helper.futureClass);
+    if (valueType == const DynamicType()) {
+      valueType = elementTypeFromReturnType(helper.futureOrClass);
     }
-    var completerTypeArguments = <DartType>[returnType];
-    var completerType =
-        new InterfaceType(helper.completerClass, completerTypeArguments);
+    final DartType returnType =
+        new InterfaceType(helper.futureOrClass, <DartType>[valueType]);
+    var completerTypeArguments = <DartType>[valueType];
 
-    // final Completer<T> :completer = new Completer<T>.sync();
-    completerVariable = new VariableDeclaration(":completer",
-        initializer: new StaticInvocation(helper.completerConstructor,
-            new Arguments([], types: completerTypeArguments))
-          ..fileOffset = enclosingFunction.body.fileOffset,
-        isFinal: true,
-        type: completerType);
+    if (syncAsync) {
+      final completerType = new InterfaceType(
+          helper.asyncAwaitCompleterClass, completerTypeArguments);
+      // final Completer<T> :async_completer = new _AsyncAwaitCompleter<T>();
+      completerVariable = new VariableDeclaration(":async_completer",
+          initializer: new ConstructorInvocation(
+              helper.asyncAwaitCompleterConstructor,
+              new Arguments([], types: completerTypeArguments))
+            ..fileOffset = enclosingFunction.body?.fileOffset ?? -1,
+          isFinal: true,
+          type: completerType);
+    } else {
+      final completerType =
+          new InterfaceType(helper.completerClass, completerTypeArguments);
+      // final Completer<T> :async_completer = new Completer<T>.sync();
+      completerVariable = new VariableDeclaration(":async_completer",
+          initializer: new StaticInvocation(helper.completerConstructor,
+              new Arguments([], types: completerTypeArguments))
+            ..fileOffset = enclosingFunction.body?.fileOffset ?? -1,
+          isFinal: true,
+          type: completerType);
+    }
     statements.add(completerVariable);
 
     returnVariable = new VariableDeclaration(":return_value", type: returnType);
@@ -812,18 +1011,27 @@ class AsyncFunctionRewriter extends AsyncRewriterBase {
 
     setupAsyncContinuations(statements);
 
-    // new Future.microtask(:async_op);
-    var newMicrotaskStatement = new ExpressionStatement(new StaticInvocation(
-        helper.futureMicrotaskConstructor,
-        new Arguments([new VariableGet(nestedClosureVariable)],
-            types: [const DynamicType()]))
-      ..fileOffset = enclosingFunction.fileOffset);
-    statements.add(newMicrotaskStatement);
-
-    // return :completer.future;
+    if (syncAsync) {
+      // :async_completer.start(:async_op);
+      var startStatement = new ExpressionStatement(new MethodInvocation(
+          new VariableGet(completerVariable),
+          new Name('start'),
+          new Arguments([new VariableGet(nestedClosureVariable)]))
+        ..fileOffset = enclosingFunction.fileOffset);
+      statements.add(startStatement);
+    } else {
+      // new Future.microtask(:async_op);
+      var newMicrotaskStatement = new ExpressionStatement(new StaticInvocation(
+          helper.futureMicrotaskConstructor,
+          new Arguments([new VariableGet(nestedClosureVariable)],
+              types: [const DynamicType()]))
+        ..fileOffset = enclosingFunction.fileOffset);
+      statements.add(newMicrotaskStatement);
+    }
+    // return :async_completer.future;
     var completerGet = new VariableGet(completerVariable);
-    var returnStatement = new ReturnStatement(
-        new PropertyGet(completerGet, new Name('future', helper.asyncLibrary)));
+    var returnStatement = new ReturnStatement(new PropertyGet(completerGet,
+        new Name('future', helper.asyncLibrary), helper.completerFuture));
     statements.add(returnStatement);
 
     enclosingFunction.body = new Block(statements);
@@ -835,11 +1043,12 @@ class AsyncFunctionRewriter extends AsyncRewriterBase {
   Statement buildCatchBody(exceptionVariable, stackTraceVariable) {
     return new ExpressionStatement(new MethodInvocation(
         new VariableGet(completerVariable),
-        new Name("completeError", helper.asyncLibrary),
+        new Name("completeError"),
         new Arguments([
           new VariableGet(exceptionVariable),
           new VariableGet(stackTraceVariable)
-        ])));
+        ]),
+        helper.completerCompleteError));
   }
 
   Statement buildReturn(Statement body) {
@@ -847,10 +1056,12 @@ class AsyncFunctionRewriter extends AsyncRewriterBase {
     // return value variable followed by a break from the labeled body.
     return new Block(<Statement>[
       body,
-      new ExpressionStatement(new MethodInvocation(
-          new VariableGet(completerVariable),
-          new Name("complete", helper.asyncLibrary),
-          new Arguments([new VariableGet(returnVariable)]))),
+      new ExpressionStatement(new StaticInvocation(
+          helper.completeOnAsyncReturn,
+          new Arguments([
+            new VariableGet(completerVariable),
+            new VariableGet(returnVariable)
+          ]))),
       new ReturnStatement()..fileOffset = enclosingFunction.fileEndOffset
     ]);
   }
@@ -867,56 +1078,124 @@ class AsyncFunctionRewriter extends AsyncRewriterBase {
 }
 
 class HelperNodes {
-  final Library asyncLibrary;
-  final Library coreLibrary;
-  final Class iteratorClass;
-  final Class futureClass;
-  final Class completerClass;
-  final Procedure printProcedure;
-  final Procedure completerConstructor;
-  final Procedure futureMicrotaskConstructor;
-  final Constructor streamControllerConstructor;
-  final Constructor syncIterableConstructor;
-  final Constructor streamIteratorConstructor;
-  final Procedure asyncThenWrapper;
   final Procedure asyncErrorWrapper;
+  final Library asyncLibrary;
+  final Procedure asyncStackTraceHelper;
+  final Member asyncStarStreamControllerAdd;
+  final Member asyncStarStreamControllerAddError;
+  final Member asyncStarStreamControllerAddStream;
+  final Class asyncStarStreamControllerClass;
+  final Member asyncStarStreamControllerClose;
+  final Constructor asyncStarStreamControllerConstructor;
+  final Member asyncStarStreamControllerStream;
+  final Procedure asyncThenWrapper;
   final Procedure awaitHelper;
+  final Class completerClass;
+  final Class asyncAwaitCompleterClass;
+  final Member completerComplete;
+  final Member completerCompleteError;
+  final Member completerConstructor;
+  final Member asyncAwaitCompleterConstructor;
+  final Member completeOnAsyncReturn;
+  final Member completerFuture;
+  final Library coreLibrary;
   final CoreTypes coreTypes;
+  final Class futureClass;
+  final Procedure futureMicrotaskConstructor;
+  final Class futureOrClass;
+  final Class iterableClass;
+  final Class iteratorClass;
+  final Procedure printProcedure;
+  final Class streamClass;
+  final Member streamIteratorCancel;
+  final Class streamIteratorClass;
+  final Constructor streamIteratorConstructor;
+  final Member streamIteratorCurrent;
+  final Member streamIteratorMoveNext;
+  final Constructor syncIterableConstructor;
+  final Class syncIteratorClass;
+  final Member syncIteratorCurrent;
+  final Member syncIteratorYieldEachIterable;
 
-  HelperNodes(
-      this.asyncLibrary,
-      this.coreLibrary,
-      this.iteratorClass,
-      this.futureClass,
-      this.completerClass,
-      this.printProcedure,
-      this.completerConstructor,
-      this.syncIterableConstructor,
-      this.streamIteratorConstructor,
-      this.futureMicrotaskConstructor,
-      this.streamControllerConstructor,
-      this.asyncThenWrapper,
+  HelperNodes._(
       this.asyncErrorWrapper,
+      this.asyncLibrary,
+      this.asyncStackTraceHelper,
+      this.asyncStarStreamControllerAdd,
+      this.asyncStarStreamControllerAddError,
+      this.asyncStarStreamControllerAddStream,
+      this.asyncStarStreamControllerClass,
+      this.asyncStarStreamControllerClose,
+      this.asyncStarStreamControllerConstructor,
+      this.asyncStarStreamControllerStream,
+      this.asyncThenWrapper,
       this.awaitHelper,
-      this.coreTypes);
+      this.completerClass,
+      this.asyncAwaitCompleterClass,
+      this.completerComplete,
+      this.completerCompleteError,
+      this.completerConstructor,
+      this.asyncAwaitCompleterConstructor,
+      this.completeOnAsyncReturn,
+      this.completerFuture,
+      this.coreLibrary,
+      this.coreTypes,
+      this.futureClass,
+      this.futureMicrotaskConstructor,
+      this.futureOrClass,
+      this.iterableClass,
+      this.iteratorClass,
+      this.printProcedure,
+      this.streamClass,
+      this.streamIteratorCancel,
+      this.streamIteratorClass,
+      this.streamIteratorConstructor,
+      this.streamIteratorCurrent,
+      this.streamIteratorMoveNext,
+      this.syncIterableConstructor,
+      this.syncIteratorClass,
+      this.syncIteratorCurrent,
+      this.syncIteratorYieldEachIterable);
 
-  factory HelperNodes.fromProgram(Program program) {
-    var coreTypes = new CoreTypes(program);
-    return new HelperNodes(
-        coreTypes.getLibrary('dart:async'),
-        coreTypes.getLibrary('dart:core'),
-        coreTypes.getClass('dart:core', 'Iterator'),
-        coreTypes.getClass('dart:async', 'Future'),
-        coreTypes.getClass('dart:async', 'Completer'),
-        coreTypes.getTopLevelMember('dart:core', 'print'),
-        coreTypes.getMember('dart:async', 'Completer', 'sync'),
-        coreTypes.getMember('dart:core', '_SyncIterable', ''),
-        coreTypes.getMember('dart:async', '_StreamIterator', ''),
-        coreTypes.getMember('dart:async', 'Future', 'microtask'),
-        coreTypes.getMember('dart:async', '_AsyncStarStreamController', ''),
-        coreTypes.getTopLevelMember('dart:async', '_asyncThenWrapperHelper'),
-        coreTypes.getTopLevelMember('dart:async', '_asyncErrorWrapperHelper'),
-        coreTypes.getTopLevelMember('dart:async', '_awaitHelper'),
-        coreTypes);
+  factory HelperNodes.fromCoreTypes(CoreTypes coreTypes) {
+    return new HelperNodes._(
+        coreTypes.asyncErrorWrapperHelperProcedure,
+        coreTypes.asyncLibrary,
+        coreTypes.asyncStackTraceHelperProcedure,
+        coreTypes.asyncStarStreamControllerAdd,
+        coreTypes.asyncStarStreamControllerAddError,
+        coreTypes.asyncStarStreamControllerAddStream,
+        coreTypes.asyncStarStreamControllerClass,
+        coreTypes.asyncStarStreamControllerClose,
+        coreTypes.asyncStarStreamControllerDefaultConstructor,
+        coreTypes.asyncStarStreamControllerStream,
+        coreTypes.asyncThenWrapperHelperProcedure,
+        coreTypes.awaitHelperProcedure,
+        coreTypes.completerClass,
+        coreTypes.asyncAwaitCompleterClass,
+        coreTypes.completerComplete,
+        coreTypes.completerCompleteError,
+        coreTypes.completerSyncConstructor,
+        coreTypes.asyncAwaitCompleterConstructor,
+        coreTypes.completeOnAsyncReturn,
+        coreTypes.completerFuture,
+        coreTypes.coreLibrary,
+        coreTypes,
+        coreTypes.futureClass,
+        coreTypes.futureMicrotaskConstructor,
+        coreTypes.futureOrClass,
+        coreTypes.iterableClass,
+        coreTypes.iteratorClass,
+        coreTypes.printProcedure,
+        coreTypes.streamClass,
+        coreTypes.streamIteratorCancel,
+        coreTypes.streamIteratorClass,
+        coreTypes.streamIteratorDefaultConstructor,
+        coreTypes.streamIteratorCurrent,
+        coreTypes.streamIteratorMoveNext,
+        coreTypes.syncIterableDefaultConstructor,
+        coreTypes.syncIteratorClass,
+        coreTypes.syncIteratorCurrent,
+        coreTypes.syncIteratorYieldEachIterable);
   }
 }
