@@ -3,9 +3,13 @@
 // BSD-style license that can be found in the LICENSE file.
 library kernel.ast.visitor;
 
+import 'dart:core' hide MapEntry;
+
 import 'ast.dart';
 
 abstract class ExpressionVisitor<R> {
+  const ExpressionVisitor();
+
   R defaultExpression(Expression node) => null;
   R defaultBasicLiteral(BasicLiteral node) => defaultExpression(node);
 
@@ -45,12 +49,14 @@ abstract class ExpressionVisitor<R> {
   R visitMapLiteral(MapLiteral node) => defaultExpression(node);
   R visitAwaitExpression(AwaitExpression node) => defaultExpression(node);
   R visitFunctionExpression(FunctionExpression node) => defaultExpression(node);
+  R visitConstantExpression(ConstantExpression node) => defaultExpression(node);
   R visitStringLiteral(StringLiteral node) => defaultBasicLiteral(node);
   R visitIntLiteral(IntLiteral node) => defaultBasicLiteral(node);
   R visitDoubleLiteral(DoubleLiteral node) => defaultBasicLiteral(node);
   R visitBoolLiteral(BoolLiteral node) => defaultBasicLiteral(node);
   R visitNullLiteral(NullLiteral node) => defaultBasicLiteral(node);
   R visitLet(Let node) => defaultExpression(node);
+  R visitInstantiation(Instantiation node) => defaultExpression(node);
   R visitLoadLibrary(LoadLibrary node) => defaultExpression(node);
   R visitCheckLibraryIsLoaded(CheckLibraryIsLoaded node) =>
       defaultExpression(node);
@@ -58,15 +64,18 @@ abstract class ExpressionVisitor<R> {
   R visitVectorGet(VectorGet node) => defaultExpression(node);
   R visitVectorSet(VectorSet node) => defaultExpression(node);
   R visitVectorCopy(VectorCopy node) => defaultExpression(node);
+  R visitClosureCreation(ClosureCreation node) => defaultExpression(node);
 }
 
 abstract class StatementVisitor<R> {
+  const StatementVisitor();
+
   R defaultStatement(Statement node) => null;
 
-  R visitInvalidStatement(InvalidStatement node) => defaultStatement(node);
   R visitExpressionStatement(ExpressionStatement node) =>
       defaultStatement(node);
   R visitBlock(Block node) => defaultStatement(node);
+  R visitAssertBlock(AssertBlock node) => defaultStatement(node);
   R visitEmptyStatement(EmptyStatement node) => defaultStatement(node);
   R visitAssertStatement(AssertStatement node) => defaultStatement(node);
   R visitLabeledStatement(LabeledStatement node) => defaultStatement(node);
@@ -90,14 +99,21 @@ abstract class StatementVisitor<R> {
 }
 
 abstract class MemberVisitor<R> {
+  const MemberVisitor();
+
   R defaultMember(Member node) => null;
 
   R visitConstructor(Constructor node) => defaultMember(node);
   R visitProcedure(Procedure node) => defaultMember(node);
   R visitField(Field node) => defaultMember(node);
+  R visitRedirectingFactoryConstructor(RedirectingFactoryConstructor node) {
+    return defaultMember(node);
+  }
 }
 
 abstract class InitializerVisitor<R> {
+  const InitializerVisitor();
+
   R defaultInitializer(Initializer node) => null;
 
   R visitInvalidInitializer(InvalidInitializer node) =>
@@ -107,6 +123,7 @@ abstract class InitializerVisitor<R> {
   R visitRedirectingInitializer(RedirectingInitializer node) =>
       defaultInitializer(node);
   R visitLocalInitializer(LocalInitializer node) => defaultInitializer(node);
+  R visitAssertInitializer(AssertInitializer node) => defaultInitializer(node);
 }
 
 class TreeVisitor<R>
@@ -115,6 +132,8 @@ class TreeVisitor<R>
         StatementVisitor<R>,
         MemberVisitor<R>,
         InitializerVisitor<R> {
+  const TreeVisitor();
+
   R defaultTreeNode(TreeNode node) => null;
 
   // Expressions
@@ -156,12 +175,14 @@ class TreeVisitor<R>
   R visitMapLiteral(MapLiteral node) => defaultExpression(node);
   R visitAwaitExpression(AwaitExpression node) => defaultExpression(node);
   R visitFunctionExpression(FunctionExpression node) => defaultExpression(node);
+  R visitConstantExpression(ConstantExpression node) => defaultExpression(node);
   R visitStringLiteral(StringLiteral node) => defaultBasicLiteral(node);
   R visitIntLiteral(IntLiteral node) => defaultBasicLiteral(node);
   R visitDoubleLiteral(DoubleLiteral node) => defaultBasicLiteral(node);
   R visitBoolLiteral(BoolLiteral node) => defaultBasicLiteral(node);
   R visitNullLiteral(NullLiteral node) => defaultBasicLiteral(node);
   R visitLet(Let node) => defaultExpression(node);
+  R visitInstantiation(Instantiation node) => defaultExpression(node);
   R visitLoadLibrary(LoadLibrary node) => defaultExpression(node);
   R visitCheckLibraryIsLoaded(CheckLibraryIsLoaded node) =>
       defaultExpression(node);
@@ -169,13 +190,14 @@ class TreeVisitor<R>
   R visitVectorGet(VectorGet node) => defaultExpression(node);
   R visitVectorSet(VectorSet node) => defaultExpression(node);
   R visitVectorCopy(VectorCopy node) => defaultExpression(node);
+  R visitClosureCreation(ClosureCreation node) => defaultExpression(node);
 
   // Statements
   R defaultStatement(Statement node) => defaultTreeNode(node);
-  R visitInvalidStatement(InvalidStatement node) => defaultStatement(node);
   R visitExpressionStatement(ExpressionStatement node) =>
       defaultStatement(node);
   R visitBlock(Block node) => defaultStatement(node);
+  R visitAssertBlock(AssertBlock node) => defaultStatement(node);
   R visitEmptyStatement(EmptyStatement node) => defaultStatement(node);
   R visitAssertStatement(AssertStatement node) => defaultStatement(node);
   R visitLabeledStatement(LabeledStatement node) => defaultStatement(node);
@@ -202,6 +224,9 @@ class TreeVisitor<R>
   R visitConstructor(Constructor node) => defaultMember(node);
   R visitProcedure(Procedure node) => defaultMember(node);
   R visitField(Field node) => defaultMember(node);
+  R visitRedirectingFactoryConstructor(RedirectingFactoryConstructor node) {
+    return defaultMember(node);
+  }
 
   // Classes
   R visitClass(Class node) => defaultTreeNode(node);
@@ -215,10 +240,14 @@ class TreeVisitor<R>
   R visitRedirectingInitializer(RedirectingInitializer node) =>
       defaultInitializer(node);
   R visitLocalInitializer(LocalInitializer node) => defaultInitializer(node);
+  R visitAssertInitializer(AssertInitializer node) => defaultInitializer(node);
 
   // Other tree nodes
   R visitLibrary(Library node) => defaultTreeNode(node);
-  R visitDeferredImport(DeferredImport node) => defaultTreeNode(node);
+  R visitLibraryDependency(LibraryDependency node) => defaultTreeNode(node);
+  R visitCombinator(Combinator node) => defaultTreeNode(node);
+  R visitLibraryPart(LibraryPart node) => defaultTreeNode(node);
+  R visitTypedef(Typedef node) => defaultTreeNode(node);
   R visitTypeParameter(TypeParameter node) => defaultTreeNode(node);
   R visitFunctionNode(FunctionNode node) => defaultTreeNode(node);
   R visitArguments(Arguments node) => defaultTreeNode(node);
@@ -226,7 +255,7 @@ class TreeVisitor<R>
   R visitSwitchCase(SwitchCase node) => defaultTreeNode(node);
   R visitCatch(Catch node) => defaultTreeNode(node);
   R visitMapEntry(MapEntry node) => defaultTreeNode(node);
-  R visitProgram(Program node) => defaultTreeNode(node);
+  R visitComponent(Component node) => defaultTreeNode(node);
 }
 
 class DartTypeVisitor<R> {
@@ -240,18 +269,47 @@ class DartTypeVisitor<R> {
   R visitVectorType(VectorType node) => defaultDartType(node);
   R visitFunctionType(FunctionType node) => defaultDartType(node);
   R visitTypeParameterType(TypeParameterType node) => defaultDartType(node);
+  R visitTypedefType(TypedefType node) => defaultDartType(node);
+}
+
+class ConstantVisitor<R> {
+  R defaultConstant(Constant node) => null;
+
+  R visitNullConstant(NullConstant node) => defaultConstant(node);
+  R visitBoolConstant(BoolConstant node) => defaultConstant(node);
+  R visitIntConstant(IntConstant node) => defaultConstant(node);
+  R visitDoubleConstant(DoubleConstant node) => defaultConstant(node);
+  R visitStringConstant(StringConstant node) => defaultConstant(node);
+  R visitMapConstant(MapConstant node) => defaultConstant(node);
+  R visitListConstant(ListConstant node) => defaultConstant(node);
+  R visitInstanceConstant(InstanceConstant node) => defaultConstant(node);
+  R visitPartialInstantiationConstant(PartialInstantiationConstant node) =>
+      defaultConstant(node);
+  R visitTearOffConstant(TearOffConstant node) => defaultConstant(node);
+  R visitTypeLiteralConstant(TypeLiteralConstant node) => defaultConstant(node);
 }
 
 class MemberReferenceVisitor<R> {
+  const MemberReferenceVisitor();
+
   R defaultMemberReference(Member node) => null;
 
   R visitFieldReference(Field node) => defaultMemberReference(node);
   R visitConstructorReference(Constructor node) => defaultMemberReference(node);
   R visitProcedureReference(Procedure node) => defaultMemberReference(node);
+  R visitRedirectingFactoryConstructorReference(
+      RedirectingFactoryConstructor node) {
+    return defaultMemberReference(node);
+  }
 }
 
 class Visitor<R> extends TreeVisitor<R>
-    implements DartTypeVisitor<R>, MemberReferenceVisitor<R> {
+    implements
+        DartTypeVisitor<R>,
+        ConstantVisitor<R>,
+        MemberReferenceVisitor<R> {
+  const Visitor();
+
   /// The catch-all case, except for references.
   R defaultNode(Node node) => null;
   R defaultTreeNode(TreeNode node) => defaultNode(node);
@@ -266,15 +324,62 @@ class Visitor<R> extends TreeVisitor<R>
   R visitVectorType(VectorType node) => defaultDartType(node);
   R visitFunctionType(FunctionType node) => defaultDartType(node);
   R visitTypeParameterType(TypeParameterType node) => defaultDartType(node);
+  R visitTypedefType(TypedefType node) => defaultDartType(node);
+
+  // Constants
+  R defaultConstant(Constant node) => defaultNode(node);
+  R visitNullConstant(NullConstant node) => defaultConstant(node);
+  R visitBoolConstant(BoolConstant node) => defaultConstant(node);
+  R visitIntConstant(IntConstant node) => defaultConstant(node);
+  R visitDoubleConstant(DoubleConstant node) => defaultConstant(node);
+  R visitStringConstant(StringConstant node) => defaultConstant(node);
+  R visitMapConstant(MapConstant node) => defaultConstant(node);
+  R visitListConstant(ListConstant node) => defaultConstant(node);
+  R visitInstanceConstant(InstanceConstant node) => defaultConstant(node);
+  R visitPartialInstantiationConstant(PartialInstantiationConstant node) =>
+      defaultConstant(node);
+  R visitTearOffConstant(TearOffConstant node) => defaultConstant(node);
+  R visitTypeLiteralConstant(TypeLiteralConstant node) => defaultConstant(node);
 
   // Class references
   R visitClassReference(Class node) => null;
+  R visitTypedefReference(Typedef node) => null;
+
+  // Constant references
+  R defaultConstantReference(Constant node) => null;
+  R visitNullConstantReference(NullConstant node) =>
+      defaultConstantReference(node);
+  R visitBoolConstantReference(BoolConstant node) =>
+      defaultConstantReference(node);
+  R visitIntConstantReference(IntConstant node) =>
+      defaultConstantReference(node);
+  R visitDoubleConstantReference(DoubleConstant node) =>
+      defaultConstantReference(node);
+  R visitStringConstantReference(StringConstant node) =>
+      defaultConstantReference(node);
+  R visitMapConstantReference(MapConstant node) =>
+      defaultConstantReference(node);
+  R visitListConstantReference(ListConstant node) =>
+      defaultConstantReference(node);
+  R visitInstanceConstantReference(InstanceConstant node) =>
+      defaultConstantReference(node);
+  R visitPartialInstantiationConstantReference(
+          PartialInstantiationConstant node) =>
+      defaultConstantReference(node);
+  R visitTearOffConstantReference(TearOffConstant node) =>
+      defaultConstantReference(node);
+  R visitTypeLiteralConstantReference(TypeLiteralConstant node) =>
+      defaultConstantReference(node);
 
   // Member references
   R defaultMemberReference(Member node) => null;
   R visitFieldReference(Field node) => defaultMemberReference(node);
   R visitConstructorReference(Constructor node) => defaultMemberReference(node);
   R visitProcedureReference(Procedure node) => defaultMemberReference(node);
+  R visitRedirectingFactoryConstructorReference(
+      RedirectingFactoryConstructor node) {
+    return defaultMemberReference(node);
+  }
 
   R visitName(Name node) => defaultNode(node);
   R visitSupertype(Supertype node) => defaultNode(node);
@@ -282,6 +387,8 @@ class Visitor<R> extends TreeVisitor<R>
 }
 
 class RecursiveVisitor<R> extends Visitor<R> {
+  const RecursiveVisitor();
+
   R defaultNode(Node node) {
     node.visitChildren(this);
     return null;
@@ -313,11 +420,15 @@ class RecursiveVisitor<R> extends Visitor<R> {
 ///     }
 ///
 class Transformer extends TreeVisitor<TreeNode> {
+  const Transformer();
+
   /// Replaces a use of a type.
   ///
   /// By default, recursion stops at this point.
   DartType visitDartType(DartType node) => node;
 
+  Constant visitConstant(Constant node) => node;
+
   Supertype visitSupertype(Supertype node) => node;
 
   TreeNode defaultTreeNode(TreeNode node) {
@@ -326,109 +437,120 @@ class Transformer extends TreeVisitor<TreeNode> {
   }
 }
 
-abstract class ExpressionVisitor1<R> {
-  R defaultExpression(Expression node, arg) => null;
-  R defaultBasicLiteral(BasicLiteral node, arg) => defaultExpression(node, arg);
-  R visitInvalidExpression(InvalidExpression node, arg) =>
+abstract class ExpressionVisitor1<R, T> {
+  const ExpressionVisitor1();
+
+  R defaultExpression(Expression node, T arg) => null;
+  R defaultBasicLiteral(BasicLiteral node, T arg) =>
       defaultExpression(node, arg);
-  R visitVariableGet(VariableGet node, arg) => defaultExpression(node, arg);
-  R visitVariableSet(VariableSet node, arg) => defaultExpression(node, arg);
-  R visitPropertyGet(PropertyGet node, arg) => defaultExpression(node, arg);
-  R visitPropertySet(PropertySet node, arg) => defaultExpression(node, arg);
-  R visitDirectPropertyGet(DirectPropertyGet node, arg) =>
+  R visitInvalidExpression(InvalidExpression node, T arg) =>
       defaultExpression(node, arg);
-  R visitDirectPropertySet(DirectPropertySet node, arg) =>
+  R visitVariableGet(VariableGet node, T arg) => defaultExpression(node, arg);
+  R visitVariableSet(VariableSet node, T arg) => defaultExpression(node, arg);
+  R visitPropertyGet(PropertyGet node, T arg) => defaultExpression(node, arg);
+  R visitPropertySet(PropertySet node, T arg) => defaultExpression(node, arg);
+  R visitDirectPropertyGet(DirectPropertyGet node, T arg) =>
       defaultExpression(node, arg);
-  R visitSuperPropertyGet(SuperPropertyGet node, arg) =>
+  R visitDirectPropertySet(DirectPropertySet node, T arg) =>
       defaultExpression(node, arg);
-  R visitSuperPropertySet(SuperPropertySet node, arg) =>
+  R visitSuperPropertyGet(SuperPropertyGet node, T arg) =>
       defaultExpression(node, arg);
-  R visitStaticGet(StaticGet node, arg) => defaultExpression(node, arg);
-  R visitStaticSet(StaticSet node, arg) => defaultExpression(node, arg);
-  R visitMethodInvocation(MethodInvocation node, arg) =>
+  R visitSuperPropertySet(SuperPropertySet node, T arg) =>
       defaultExpression(node, arg);
-  R visitDirectMethodInvocation(DirectMethodInvocation node, arg) =>
+  R visitStaticGet(StaticGet node, T arg) => defaultExpression(node, arg);
+  R visitStaticSet(StaticSet node, T arg) => defaultExpression(node, arg);
+  R visitMethodInvocation(MethodInvocation node, T arg) =>
       defaultExpression(node, arg);
-  R visitSuperMethodInvocation(SuperMethodInvocation node, arg) =>
+  R visitDirectMethodInvocation(DirectMethodInvocation node, T arg) =>
       defaultExpression(node, arg);
-  R visitStaticInvocation(StaticInvocation node, arg) =>
+  R visitSuperMethodInvocation(SuperMethodInvocation node, T arg) =>
       defaultExpression(node, arg);
-  R visitConstructorInvocation(ConstructorInvocation node, arg) =>
+  R visitStaticInvocation(StaticInvocation node, T arg) =>
       defaultExpression(node, arg);
-  R visitNot(Not node, arg) => defaultExpression(node, arg);
-  R visitLogicalExpression(LogicalExpression node, arg) =>
+  R visitConstructorInvocation(ConstructorInvocation node, T arg) =>
       defaultExpression(node, arg);
-  R visitConditionalExpression(ConditionalExpression node, arg) =>
+  R visitNot(Not node, T arg) => defaultExpression(node, arg);
+  R visitLogicalExpression(LogicalExpression node, T arg) =>
       defaultExpression(node, arg);
-  R visitStringConcatenation(StringConcatenation node, arg) =>
+  R visitConditionalExpression(ConditionalExpression node, T arg) =>
       defaultExpression(node, arg);
-  R visitIsExpression(IsExpression node, arg) => defaultExpression(node, arg);
-  R visitAsExpression(AsExpression node, arg) => defaultExpression(node, arg);
-  R visitSymbolLiteral(SymbolLiteral node, arg) => defaultExpression(node, arg);
-  R visitTypeLiteral(TypeLiteral node, arg) => defaultExpression(node, arg);
-  R visitThisExpression(ThisExpression node, arg) =>
+  R visitStringConcatenation(StringConcatenation node, T arg) =>
       defaultExpression(node, arg);
-  R visitRethrow(Rethrow node, arg) => defaultExpression(node, arg);
-  R visitThrow(Throw node, arg) => defaultExpression(node, arg);
-  R visitListLiteral(ListLiteral node, arg) => defaultExpression(node, arg);
-  R visitMapLiteral(MapLiteral node, arg) => defaultExpression(node, arg);
-  R visitAwaitExpression(AwaitExpression node, arg) =>
+  R visitIsExpression(IsExpression node, T arg) => defaultExpression(node, arg);
+  R visitAsExpression(AsExpression node, T arg) => defaultExpression(node, arg);
+  R visitSymbolLiteral(SymbolLiteral node, T arg) =>
       defaultExpression(node, arg);
-  R visitFunctionExpression(FunctionExpression node, arg) =>
+  R visitTypeLiteral(TypeLiteral node, T arg) => defaultExpression(node, arg);
+  R visitThisExpression(ThisExpression node, T arg) =>
       defaultExpression(node, arg);
-  R visitStringLiteral(StringLiteral node, arg) =>
+  R visitConstantExpression(ConstantExpression node, arg) =>
+      defaultExpression(node, arg);
+  R visitRethrow(Rethrow node, T arg) => defaultExpression(node, arg);
+  R visitThrow(Throw node, T arg) => defaultExpression(node, arg);
+  R visitListLiteral(ListLiteral node, T arg) => defaultExpression(node, arg);
+  R visitMapLiteral(MapLiteral node, T arg) => defaultExpression(node, arg);
+  R visitAwaitExpression(AwaitExpression node, T arg) =>
+      defaultExpression(node, arg);
+  R visitFunctionExpression(FunctionExpression node, T arg) =>
+      defaultExpression(node, arg);
+  R visitIntLiteral(IntLiteral node, T arg) => defaultBasicLiteral(node, arg);
+  R visitStringLiteral(StringLiteral node, T arg) =>
       defaultBasicLiteral(node, arg);
-  R visitIntLiteral(IntLiteral node, arg) => defaultBasicLiteral(node, arg);
-  R visitDoubleLiteral(DoubleLiteral node, arg) =>
+  R visitDoubleLiteral(DoubleLiteral node, T arg) =>
       defaultBasicLiteral(node, arg);
-  R visitBoolLiteral(BoolLiteral node, arg) => defaultBasicLiteral(node, arg);
-  R visitNullLiteral(NullLiteral node, arg) => defaultBasicLiteral(node, arg);
-  R visitLet(Let node, arg) => defaultExpression(node, arg);
-  R visitLoadLibrary(LoadLibrary node, arg) => defaultExpression(node, arg);
-  R visitCheckLibraryIsLoaded(CheckLibraryIsLoaded node, arg) =>
+  R visitBoolLiteral(BoolLiteral node, T arg) => defaultBasicLiteral(node, arg);
+  R visitNullLiteral(NullLiteral node, T arg) => defaultBasicLiteral(node, arg);
+  R visitLet(Let node, T arg) => defaultExpression(node, arg);
+  R visitInstantiation(Instantiation node, T arg) =>
       defaultExpression(node, arg);
-  R visitVectorCreation(VectorCreation node, arg) =>
+  R visitLoadLibrary(LoadLibrary node, T arg) => defaultExpression(node, arg);
+  R visitCheckLibraryIsLoaded(CheckLibraryIsLoaded node, T arg) =>
+      defaultExpression(node, arg);
+  R visitVectorCreation(VectorCreation node, T arg) =>
+      defaultExpression(node, arg);
+  R visitVectorGet(VectorGet node, T arg) => defaultExpression(node, arg);
+  R visitVectorSet(VectorSet node, T arg) => defaultExpression(node, arg);
+  R visitVectorCopy(VectorCopy node, T arg) => defaultExpression(node, arg);
+  R visitClosureCreation(ClosureCreation node, T arg) =>
       defaultExpression(node, arg);
-  R visitVectorGet(VectorGet node, arg) => defaultExpression(node, arg);
-  R visitVectorSet(VectorSet node, arg) => defaultExpression(node, arg);
-  R visitVectorCopy(VectorCopy node, arg) => defaultExpression(node, arg);
 }
 
-abstract class StatementVisitor1<R> {
-  R defaultStatement(Statement node, arg) => null;
+abstract class StatementVisitor1<R, T> {
+  const StatementVisitor1();
 
-  R visitInvalidStatement(InvalidStatement node, arg) =>
+  R defaultStatement(Statement node, T arg) => null;
+
+  R visitExpressionStatement(ExpressionStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitExpressionStatement(ExpressionStatement node, arg) =>
+  R visitBlock(Block node, T arg) => defaultStatement(node, arg);
+  R visitAssertBlock(AssertBlock node, T arg) => defaultStatement(node, arg);
+  R visitEmptyStatement(EmptyStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitBlock(Block node, arg) => defaultStatement(node, arg);
-  R visitEmptyStatement(EmptyStatement node, arg) =>
+  R visitAssertStatement(AssertStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitAssertStatement(AssertStatement node, arg) =>
+  R visitLabeledStatement(LabeledStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitLabeledStatement(LabeledStatement node, arg) =>
+  R visitBreakStatement(BreakStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitBreakStatement(BreakStatement node, arg) =>
+  R visitWhileStatement(WhileStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitWhileStatement(WhileStatement node, arg) =>
+  R visitDoStatement(DoStatement node, T arg) => defaultStatement(node, arg);
+  R visitForStatement(ForStatement node, T arg) => defaultStatement(node, arg);
+  R visitForInStatement(ForInStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitDoStatement(DoStatement node, arg) => defaultStatement(node, arg);
-  R visitForStatement(ForStatement node, arg) => defaultStatement(node, arg);
-  R visitForInStatement(ForInStatement node, arg) =>
+  R visitSwitchStatement(SwitchStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitSwitchStatement(SwitchStatement node, arg) =>
+  R visitContinueSwitchStatement(ContinueSwitchStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitContinueSwitchStatement(ContinueSwitchStatement node, arg) =>
+  R visitIfStatement(IfStatement node, T arg) => defaultStatement(node, arg);
+  R visitReturnStatement(ReturnStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitIfStatement(IfStatement node, arg) => defaultStatement(node, arg);
-  R visitReturnStatement(ReturnStatement node, arg) =>
+  R visitTryCatch(TryCatch node, T arg) => defaultStatement(node, arg);
+  R visitTryFinally(TryFinally node, T arg) => defaultStatement(node, arg);
+  R visitYieldStatement(YieldStatement node, T arg) =>
       defaultStatement(node, arg);
-  R visitTryCatch(TryCatch node, arg) => defaultStatement(node, arg);
-  R visitTryFinally(TryFinally node, arg) => defaultStatement(node, arg);
-  R visitYieldStatement(YieldStatement node, arg) =>
+  R visitVariableDeclaration(VariableDeclaration node, T arg) =>
       defaultStatement(node, arg);
-  R visitVariableDeclaration(VariableDeclaration node, arg) =>
-      defaultStatement(node, arg);
-  R visitFunctionDeclaration(FunctionDeclaration node, arg) =>
+  R visitFunctionDeclaration(FunctionDeclaration node, T arg) =>
       defaultStatement(node, arg);
 }
